from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper
from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC


class SendFeedbackTest(LiveTornadoTestCase, SeleniumHelper):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super().tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user = self.create_user(
            username="Yeti", email="yeti@snowman.com", passtext="otter1"
        )

    def test_sendfeedback(self):
        self.driver.get(self.base_url)
        self.driver.find_element(By.CSS_SELECTOR, ".feedback-tab").click()
        self.driver.find_element(By.ID, "message").click()
        self.driver.find_element(By.ID, "message").send_keys(
            "I found a problem."
        )
        self.driver.find_element(By.ID, "feedbackbutton").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, "response-message"))
        )
        assert (
            self.driver.find_element(By.ID, "response-message").text
            == "Thank you for your report!"
        )
        self.driver.find_element(By.ID, "close-feedback").click()
        self.driver.find_element(By.ID, "id-login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id-password").send_keys("otter1")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.ID, "preferences-btn"))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".feedback-tab").click()
        self.driver.find_element(By.ID, "message").click()
        self.driver.find_element(By.ID, "message").send_keys(
            "I found another problem."
        )
        self.driver.find_element(By.ID, "feedbackbutton").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, "response-message"))
        )
        assert (
            self.driver.find_element(By.ID, "response-message").text
            == "Thank you for your report!"
        )
        self.driver.find_element(By.ID, "close-feedback").click()
        self.driver.find_element(By.ID, "preferences-btn").click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Log out"]'
        ).click()



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^save/$", views.save, name="usermedia_save"),
    re_path("^delete/$", views.delete, name="usermedia_delete"),
    re_path("^images/$", views.images, name="usermedia_images"),
    re_path(
        "^save_category/$", views.save_category, name="usermedia_save_category"
    ),
]



# Generated by Django 3.1.4 on 2021-02-05 07:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("document", "0008_fix_fidus_3_3_table_header"),
    ]

    operations = [
        migrations.AddField(
            model_name="document",
            name="path",
            field=models.TextField(blank=True, default=""),
        ),
    ]



import shutil
from subprocess import call
from base.management import BaseCommand
from django.conf import settings


class Command(BaseCommand):
    help = "Check Python files for PEP8 compliance"

    def add_arguments(self, parser):
        parser.add_argument(
            "--fix",
            action="store_true",
            dest="fix",
            default=False,
            help="Whether to also attempt to fix files",
        )

    def handle(self, *args, **options):
        self.stdout.write("Linting Python code...")
        shutil.os.chdir(settings.PROJECT_PATH)
        if options["fix"]:
            call(["black", "--line-length", "79", "."])
        flake_return_value = call(
            [
                "flake8",
                "--extend-ignore",
                "E203,E501",
                "--exclude",
                "node_modules/*,venv/*,.direnv/*,configuration.py",
                "./",
            ]
        )
        if flake_return_value > 0:
            exit(flake_return_value)
        black_return_value = call(
            ["black", "--line-length", "79", ".", "--check"]
        )
        if black_return_value > 0:
            exit(black_return_value)



import json
from allauth.socialaccount.models import SocialToken

from . import models
from . import github_proxy
from . import gitlab_proxy

ALLOWED_METHODS = [
    "GET",
]


async def prepare(proxy_connector, path_parts, user):
    path_part = path_parts.pop(0) if len(path_parts) else None
    if path_part == "repos":
        await get_repos(proxy_connector, path_parts, user)


async def get_repos(proxy_connector, path_parts, user):
    if proxy_connector.request.method not in ALLOWED_METHODS:
        proxy_connector.set_status(405)
        return
    path_part = path_parts.pop(0) if len(path_parts) else None
    if path_part == "reload":
        reload = True
    else:
        reload = False
    social_tokens = {
        "github": SocialToken.objects.filter(
            account__user=user, account__provider="github"
        ).first(),
        "gitlab": SocialToken.objects.filter(
            account__user=user, account__provider="gitlab"
        ).first(),
    }

    if not social_tokens["github"] and not social_tokens["gitlab"]:
        proxy_connector.set_status(401)
        return
    repo_info = models.RepoInfo.objects.filter(user=user).first()
    if repo_info:
        if reload:
            repo_info.delete()
        else:
            proxy_connector.set_status(200)
            proxy_connector.write(json.dumps(repo_info.content))
            return
    repos = []
    if social_tokens["github"]:
        repos += await github_proxy.get_repos(
            proxy_connector, social_tokens["github"]
        )
    if social_tokens["gitlab"]:
        repos += await gitlab_proxy.get_repos(
            proxy_connector, social_tokens["gitlab"]
        )
    repo_info, created = models.RepoInfo.objects.get_or_create(user=user)
    repo_info.content = repos
    repo_info.save()
    proxy_connector.set_status(200)
    proxy_connector.write(json.dumps(repo_info.content))



# Source: https://github.com/plter/tornado_asgi_handler plter (MIT)

from tornado.web import RequestHandler

GLOBAL_CHARSET = "utf-8"


class AsgiHandler(RequestHandler):
    def initialize(self, asgi_app) -> None:
        super().initialize()
        self._asgi_app = asgi_app

    async def handle_request(self):
        headers = []
        for k in self.request.headers:
            for v in self.request.headers.get_list(k):
                headers.append(
                    (
                        k.encode(GLOBAL_CHARSET).lower(),
                        v.encode(GLOBAL_CHARSET),
                    )
                )

        scope = {
            "type": "http",
            "http_version": self.request.version,
            "path": self.request.path,
            "method": self.request.method,
            "query_string": self.request.query.encode(GLOBAL_CHARSET),
            "headers": headers,
            "client": (self.request.remote_ip, 0),
        }

        async def receive():
            return {
                "body": self.request.body,
                "type": "http.request",
                "more_body": False,
            }

        async def send(data):
            if data["type"] == "http.response.start":
                self.set_status(data["status"])
                self.clear_header("content-type")
                self.clear_header("server")
                self.clear_header("date")
                for h in data["headers"]:
                    if len(h) == 2:
                        self.add_header(
                            h[0].decode(GLOBAL_CHARSET),
                            h[1].decode(GLOBAL_CHARSET),
                        )
            elif data["type"] == "http.response.body":
                status = self.get_status()
                if status not in (204, 304) and "body" in data:
                    self.write(data["body"])
            else:
                raise RuntimeError(
                    f"Unsupported response type \"{data['type']}\" for asgi app"
                )

        await self._asgi_app(scope, receive, send)

    async def get(self):
        await self.handle_request()

    async def post(self):
        await self.handle_request()



import json
from django.db import migrations, models, transaction


def text_to_json(apps, schema_editor):
    UserImage = apps.get_model("usermedia", "UserImage")
    uimages = UserImage.objects.all().iterator()
    with transaction.atomic():
        for image in uimages:
            try:
                image.cats = json.loads(image.image_cat)
            except ValueError:
                pass
            try:
                image.copyright = json.loads(image.copyright_text)
            except ValueError:
                image.copyright = {
                    "holder": False,
                    "year": False,
                    "freeToRead": True,
                    "licenses": [],
                }
            image.save()
    DocumentImage = apps.get_model("usermedia", "DocumentImage")
    dimages = DocumentImage.objects.all().iterator()
    with transaction.atomic():
        for image in dimages:
            try:
                image.copyright = json.loads(image.copyright_text)
            except ValueError:
                image.copyright = {
                    "holder": False,
                    "year": False,
                    "freeToRead": True,
                    "licenses": [],
                }
            image.save()


def json_to_text(apps, schema_editor):
    UserImage = apps.get_model("usermedia", "UserImage")
    images = UserImage.objects.all().iterator()
    with transaction.atomic():
        for image in images:
            image.image_cat = json.dumps(image.cats)
            image.copyright_text = json.dumps(image.copyright)
            image.save()
    DocumentImage = apps.get_model("usermedia", "DocumentImage")
    dimages = DocumentImage.objects.all().iterator()
    with transaction.atomic():
        for image in dimages:
            image.copyright_text = json.dumps(image.copyright)
            image.save()


class Migration(migrations.Migration):
    atomic = False  # to prevent migrations running in single transaction
    dependencies = [
        ("usermedia", "0004_auto_20200205_2347"),
    ]

    operations = [
        migrations.AddField(
            model_name="userimage",
            name="cats",
            field=models.JSONField(default=list),
        ),
        migrations.RenameField(
            model_name="userimage",
            old_name="copyright",
            new_name="copyright_text",
        ),
        migrations.AddField(
            model_name="userimage",
            name="copyright",
            field=models.JSONField(default=dict),
        ),
        migrations.RenameField(
            model_name="documentimage",
            old_name="copyright",
            new_name="copyright_text",
        ),
        migrations.AddField(
            model_name="documentimage",
            name="copyright",
            field=models.JSONField(default=dict),
        ),
        migrations.RunPython(text_to_json, json_to_text),
        migrations.RemoveField(
            model_name="userimage",
            name="image_cat",
        ),
        migrations.RemoveField(
            model_name="userimage",
            name="copyright_text",
        ),
        migrations.RemoveField(
            model_name="documentimage",
            name="copyright_text",
        ),
    ]



from django.db import models
from django.conf import settings

from . import emails


class Feedback(models.Model):
    message = models.TextField()
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        on_delete=models.deletion.CASCADE,
    )

    def __str__(self):
        if self.owner:
            return f"{self.owner.username}: f{self.message}"
        else:
            return f"Anonymous: {self.message}"

    def save(self, *args, **kwargs):
        reply_to = ""
        if self.owner:
            from_sender = self.owner.username
            if self.owner.email:
                reply_to = f" ({self.owner.email})"
        else:
            from_sender = "Anonymous"
        emails.send_feedback(from_sender, reply_to, self.message)
        super().save(*args, **kwargs)



from datetime import datetime, timedelta

from django.conf import settings

from tornado.web import RequestHandler, StaticFileHandler, HTTPError


class HelloHandler(RequestHandler):
    def head(self):
        self.finish()

    def get(self):
        self.write("Hello from tornado")


class RobotsHandler(RequestHandler):
    def head(self):
        self.finish()

    def get(self):
        self.write(("User-agent: *\nDisallow: /*\nAllow: /$"))


class DjangoStaticFilesHandler(StaticFileHandler):
    def initialize(self, default_filename=None):
        super().initialize(None, default_filename=None)

    def validate_absolute_path(self, root, absolute_path):
        if absolute_path is False:
            raise HTTPError(404)
        return absolute_path

    def get_absolute_path(self, root, path):
        for finder in settings.STATICFILES_FINDERS:
            finderClass = self.get_class(finder)
            instance = finderClass()

            new_path = instance.find(path)
            if len(new_path) > 0:
                return new_path
        return False

    def get_class(self, kls):
        parts = kls.split(".")
        module = ".".join(parts[:-1])
        m = __import__(module)
        for comp in parts[1:]:
            m = getattr(m, comp)
        return m

    def set_extra_headers(self, path):
        if settings.DEBUG:
            self.set_header(
                "Cache-Control",
                "no-store, no-cache, must-revalidate, max-age=0",
            )
            self.set_header("Expires", "0")
            expiration = datetime.now() - timedelta(days=366)
            self.set_header("Last-Modified", expiration)
        else:
            self.set_header(
                "Expires", datetime.utcnow() + timedelta(days=365 * 10)
            )
            self.set_header(
                "Cache-Control", "max-age=" + str(86400 * 365 * 10)
            )
            self.set_header("Last-Modified", datetime.utcnow())


class SetupStaticFilesHandler(StaticFileHandler):
    def set_extra_headers(self, path):
        self.set_header(
            "Cache-Control", "no-store, no-cache, must-revalidate, max-age=0"
        )
        self.set_header("Expires", "0")
        expiration = datetime.now() - timedelta(days=366)
        self.set_header("Last-Modified", expiration)

    def compute_etag(self):
        return None

    @classmethod
    def get_absolute_path(cls, root, path):
        # all css and other resource files, but no html files.
        split_path = path.split(".")
        if len(split_path) == 1 or split_path[1] == "html":
            adjusted_path = "index.html"
        else:
            adjusted_path = path
        return super().get_absolute_path(root, adjusted_path)



import uuid

from django.db import models
from django.utils.translation import gettext as _
from django.contrib.contenttypes.fields import GenericRelation
from django.contrib.auth.models import AbstractUser
from avatar.utils import get_default_avatar_url

from document.models import AccessRight


def auto_avatar(username):
    hash = 0
    for ch in username:
        hash = ord(ch) + ((hash << 5) - hash)

    r = str((hash >> (0 * 8)) & 255)
    g = str((hash >> (1 * 8)) & 255)
    b = str((hash >> (2 * 8)) & 255)

    cl = f"rgb({r},{g},{b})"
    return {
        "url": get_default_avatar_url(),
        "uploaded": False,
        "html": (
            f'<span class="fw-string-avatar" style="background-color: {cl};">'
            f"<span>{username[0]}</span>"
            "</span>"
        ),
    }


class User(AbstractUser):
    contacts = models.ManyToManyField("self", blank=True, symmetrical=True)
    document_rights = GenericRelation(
        "document.AccessRight",
        content_type_field="holder_type",
        object_id_field="holder_id",
        related_query_name="user",
    )

    @property
    def readable_name(self):
        readable_name = self.get_full_name()
        if readable_name == "":
            readable_name = self.username
        return readable_name


class UserInvite(models.Model):
    key = models.UUIDField(
        unique=True,
        default=uuid.uuid4,
    )
    email = models.EmailField(_("email address"))
    username = models.CharField(
        max_length=150,
    )
    by = models.ForeignKey(
        User,
        related_name="invites_by",
        on_delete=models.CASCADE,
    )
    to = models.ForeignKey(
        User,
        related_name="invites_to",
        on_delete=models.CASCADE,
        blank=True,
        null=True,
    )
    document_rights = GenericRelation(
        "document.AccessRight",
        content_type_field="holder_type",
        object_id_field="holder_id",
        related_query_name="userinvite",
    )
    _apply = False

    @property
    def avatar_url(self):
        return auto_avatar(self.username)

    @property
    def readable_name(self):
        return self.username

    def get_relative_url(self):
        return "/invite/%s/" % self.key

    def apply(self):
        if not self.to:
            # Cannot apply
            return
        for right in self.document_rights.all():
            old_ar = AccessRight.objects.filter(
                user=self.to, document=right.document
            ).first()
            if old_ar:
                # If the user already has rights, we should only be upgrading
                # them, not downgrade. Unfortuantely it is not easy to
                # say how each right compares. So unless the invite gives read
                # access, or the user already has write access, we change to
                # the access right of the invite.
                if right.rights == "read":
                    pass
                elif old_ar.rights == "write":
                    pass
                else:
                    old_ar.rights = right.rights
                    old_ar.save()
            elif right.document.owner == self.to:
                pass
            else:
                right.holder_obj = self.to
                right.save()
        if self.to not in list(self.by.contacts.all()):
            self.by.contacts.add(self.to)
        self._apply = True
        self.delete()

    def __str__(self):
        return f"{self.to or self.username} by {self.by}"


class LoginAs(models.Model):
    class Meta:
        managed = False
        default_permissions = ()

        permissions = (("can_login_as", _("Can login as another user")),)



from builtins import object
import re
import os
import time


from allauth.account.models import EmailAddress
from django.contrib.auth.hashers import make_password
from django.contrib.auth import get_user_model


class SeleniumHelper(object):
    """
    Methods for manipulating django and the browser for testing purposes.
    """

    def find_urls(self, string):
        return re.findall(
            (
                "http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*(),]|"
                "(?:%[0-9a-fA-F][0-9a-fA-F]))+"
            ),
            string,
        )

    # create django data
    def create_user(
        self, username="User", email="test@example.com", passtext="p4ssw0rd"
    ):
        User = get_user_model()
        user = User.objects.create(
            username=username,
            email=email,
            password=make_password(passtext),
            is_active=True,
        )
        user.save()

        # avoid the unverified-email login trap
        EmailAddress.objects.create(
            user=user, email=email, verified=True, primary=True
        ).save()

        return user

    # drive browser
    def login_user(self, user, driver, client):
        client.force_login(user=user)
        cookie = client.cookies["sessionid"]
        if driver.current_url == "data:,":
            # To set the cookie at the right domain we load the front page.
            driver.get("%s%s" % (self.live_server_url, "/"))
        driver.add_cookie(
            {
                "name": "sessionid",
                "value": cookie.value,
                "secure": False,
                "path": "/",
            }
        )

    def logout_user(self, driver, client):
        client.logout()
        driver.delete_cookie("sessionid")

    def wait_until_file_exists(self, path, wait_time):
        count = 0
        while not os.path.exists(path):
            time.sleep(1)
            count += 1
            if count > wait_time:
                break



import time
import os
from tempfile import mkdtemp

from selenium.webdriver.common.by import By
from selenium.webdriver.support.wait import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper

from django.core import mail

from website import models


class WebsiteTest(LiveTornadoTestCase, SeleniumHelper):
    fixtures = [
        "initial_documenttemplates.json",
        "initial_styles.json",
    ]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        cls.download_dir = mkdtemp()
        driver_data = cls.get_drivers(1, cls.download_dir)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        os.rmdir(cls.download_dir)
        super().tearDownClass()

    def setUp(self):
        self.user = self.create_user(
            username="user",
            email="user@sciencenewsportal.com",
            passtext="otter1",
        )
        self.editor = self.create_user(
            username="editor",
            email="editor@sciencenewsportal.com",
            passtext="otter1",
        )
        models.Editor.objects.create(user=self.editor)

    def test_website(self):
        self.login_user(self.user, self.driver, self.client)
        self.driver.get(self.base_url + "/documents/")
        # Create news article 1
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".new_document button")
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, "editor-toolbar"))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "News article 1"
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Submit for publishing to website']",
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        submission_message = "This article is ready for publication."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(submission_message)
        emails_before_submission = len(mail.outbox)
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        # Check that email has been sent to editor
        emails_after_submission = len(mail.outbox)
        assert emails_after_submission == (emails_before_submission + 1)
        notify_editor_email = mail.outbox[-1]
        assert self.editor.email in notify_editor_email.to
        assert submission_message in notify_editor_email.body
        assert "submitted to be published" in notify_editor_email.body

        # Log in as editor and ask for changes.
        self.login_user(self.editor, self.driver, self.client)
        document_link = self.find_urls(notify_editor_email.body)[0]
        self.driver.get(document_link)
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Publish, reject or request changes']",
        ).click()
        review_message = self.driver.find_element(
            By.CSS_SELECTOR, "#submission-dialog"
        )
        assert submission_message in review_message.text
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        editor_message = "It's good, but not quite good enough."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(editor_message)
        emails_before_ask_for_changes = len(mail.outbox)
        self.driver.find_elements(By.CSS_SELECTOR, "button.fw-dark")[1].click()
        time.sleep(1)
        # Check that email has been sent to user
        emails_after_ask_for_changes = len(mail.outbox)
        assert emails_after_ask_for_changes == (
            emails_before_ask_for_changes + 1
        )
        notify_user_email = mail.outbox[-1]
        assert self.user.email in notify_user_email.to
        assert editor_message in notify_user_email.body
        assert "need to change some things" in notify_user_email.body

        # Log in as user, make changes and resubmit.
        self.login_user(self.user, self.driver, self.client)
        document_link = self.find_urls(notify_user_email.body)[0]
        self.driver.get(document_link)
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "An updated body."
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Submit for publishing to website']",
        ).click()
        review_message = self.driver.find_element(
            By.CSS_SELECTOR, "#submission-dialog"
        )
        assert submission_message in review_message.text
        assert editor_message in review_message.text
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        resubmission_message = "I've made substantial changes. Please review."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(resubmission_message)
        emails_before_resubmission = len(mail.outbox)
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        # Check that email has been sent to editor
        emails_after_resubmission = len(mail.outbox)
        assert emails_after_resubmission == (emails_before_resubmission + 1)
        notify_editor_again_email = mail.outbox[-1]
        assert self.editor.email in notify_editor_again_email.to
        assert resubmission_message in notify_editor_again_email.body
        assert "submitted to be published" in notify_editor_again_email.body

        # Log in as editor and reject.
        self.login_user(self.editor, self.driver, self.client)
        document_link = self.find_urls(notify_editor_email.body)[0]
        self.driver.get(document_link)
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Publish, reject or request changes']",
        ).click()
        review_message = self.driver.find_element(
            By.CSS_SELECTOR, "#submission-dialog"
        )
        assert submission_message in review_message.text
        assert editor_message in review_message.text
        assert resubmission_message in review_message.text
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        rejection_message = "I don't think you realize what a serious we are. This submission has been rejected."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(rejection_message)
        emails_before_rejection = len(mail.outbox)
        self.driver.find_elements(By.CSS_SELECTOR, "button.fw-dark")[2].click()
        time.sleep(1)
        # Check that email has been sent to editor
        emails_after_rejection = len(mail.outbox)
        assert emails_after_rejection == (emails_before_rejection + 1)
        notify_user_again_email = mail.outbox[-1]
        assert self.user.email in notify_user_again_email.to
        assert rejection_message in notify_user_again_email.body
        assert "reviewed and rejected" in notify_user_again_email.body

        # Check that article does not show on front page
        self.logout_user(self.driver, self.client)
        self.driver.get(self.base_url + "/")
        articles_shown = self.driver.find_elements(
            By.CSS_SELECTOR, "div.articles a.article"
        )
        assert len(articles_shown) == 0

        # Write a second article.
        self.login_user(self.user, self.driver, self.client)
        self.driver.get(self.base_url + "/documents/")
        # Create news article 1
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".new_document button")
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, "editor-toolbar"))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "News article 2"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "This article has a real body."
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Submit for publishing to website']",
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        submission_message = "This article is ready for publication and much better than the previous one."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(submission_message)
        emails_before_submission = len(mail.outbox)
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        # Check that email has been sent to editor
        emails_after_submission = len(mail.outbox)
        assert emails_after_submission == (emails_before_submission + 1)
        notify_editor_email = mail.outbox[-1]
        assert self.editor.email in notify_editor_email.to
        assert submission_message in notify_editor_email.body
        assert "submitted to be published" in notify_editor_email.body

        # Log in as editor and publish.
        self.login_user(self.editor, self.driver, self.client)
        document_link = self.find_urls(notify_editor_email.body)[0]
        self.driver.get(document_link)
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Publish, reject or request changes']",
        ).click()
        review_message = self.driver.find_element(
            By.CSS_SELECTOR, "#submission-dialog"
        )
        assert submission_message in review_message.text
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).click()
        editor_message = "It's good, really good. I'll publish it right away."
        self.driver.find_element(
            By.CSS_SELECTOR, "textarea#submission-message"
        ).send_keys(editor_message)
        emails_before_publishing = len(mail.outbox)
        self.driver.find_elements(By.CSS_SELECTOR, "button.fw-dark")[0].click()
        time.sleep(1)
        # Check that email has been sent to user
        emails_after_publishing = len(mail.outbox)
        assert emails_after_publishing == (emails_before_publishing + 1)
        notify_user_email = mail.outbox[-1]
        assert self.user.email in notify_user_email.to
        assert editor_message in notify_user_email.body

        # Check that article does show on front page
        self.logout_user(self.driver, self.client)
        self.driver.get(self.base_url + "/")
        articles_shown = self.driver.find_elements(
            By.CSS_SELECTOR, "div.articles a.article"
        )
        assert len(articles_shown) == 1

        # Log in as editor and publish directly.
        self.login_user(self.editor, self.driver, self.client)
        self.driver.get(self.base_url + "/documents/")
        # Create news article 1
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".new_document button")
            )
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, "editor-toolbar"))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "News article 3"
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "span.header-nav-item[title='Publish to website']"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "span.fw-pulldown-item[title='Publish, reject or request changes']",
        ).click()
        emails_before_publishing = len(mail.outbox)
        self.driver.find_elements(By.CSS_SELECTOR, "button.fw-dark")[0].click()
        time.sleep(1)
        # Check that no email has been sent to user
        emails_after_publishing = len(mail.outbox)
        assert emails_after_publishing == emails_before_publishing

        # Check that article does show on front page
        self.logout_user(self.driver, self.client)
        self.driver.get(self.base_url + "/")
        articles_shown = self.driver.find_elements(
            By.CSS_SELECTOR, "div.articles a.article"
        )
        assert len(articles_shown) == 2



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^flatpage/$", views.flatpage, name="flatpage"),
    re_path("^configuration/$", views.configuration, name="configuration"),
]



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^get/$", views.get_template, name="document_template_get"),
    re_path("^list/$", views.list, name="document_template_list"),
    re_path("^save/$", views.save, name="document_template_save"),
    re_path("^create/$", views.create, name="document_template_create"),
    re_path("^copy/$", views.copy, name="document_template_copy"),
    re_path("^delete/$", views.delete, name="document_template_delete"),
]



# Generated by Django 3.1.4 on 2021-06-07 06:57

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("document", "0016_delete_accessrightinvite"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accessright",
            name="rights",
            field=models.CharField(
                choices=[
                    ("write", "Writer"),
                    ("write-tracked", "Write with tracked changes"),
                    ("comment", "Commentator"),
                    (
                        "review-tracked",
                        "Reviewer who can write with tracked changes",
                    ),
                    ("review", "Reviewer"),
                    ("read", "Reader"),
                    ("read-without-comments", "Reader without comment access"),
                ],
                max_length=21,
            ),
        ),
    ]



# Generated by Django 3.1.4 on 2021-04-02 20:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0004_auto_20210402_2049"),
    ]

    operations = [
        migrations.DeleteModel(
            name="TeamMember",
        ),
    ]



import time
import os
from tempfile import mkdtemp

from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.wait import WebDriverWait
from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper


class UserTemplateManagerTest(LiveTornadoTestCase, SeleniumHelper):
    fixtures = ["initial_documenttemplates.json", "initial_styles.json"]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        cls.download_dir = mkdtemp()
        driver_data = cls.get_drivers(1, cls.download_dir)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        os.rmdir(cls.download_dir)
        super().tearDownClass()

    def setUp(self):
        self.user = self.create_user(
            username="Yeti", email="yeti@snowman.com", passtext="otter1"
        )
        self.login_user(self.user, self.driver, self.client)

    def test_template_export_import(self):
        driver = self.driver
        driver.get(f"{self.base_url}/templates/")
        templates = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr .far.fa-file"
        )
        self.assertEqual(len(templates), 1)
        editable_templates = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr .fw-data-table-title a"
        )
        self.assertEqual(len(editable_templates), 0)
        self.driver.find_element(By.CSS_SELECTOR, ".entry-select").click()
        # Try to delete the default template - should fail
        self.driver.find_element(By.CSS_SELECTOR, ".dt-bulk-dropdown").click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'li.content-menu-item[data-index="0"]'
        ).click()
        alert_element = WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-error"))
        )
        self.assertEqual(alert_element.is_displayed(), True)
        # Duplicate default template
        self.driver.find_element(By.CSS_SELECTOR, ".dt-bulk-dropdown").click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'li.content-menu-item[data-index="1"]'
        ).click()
        time.sleep(1)
        editable_templates = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr .fw-data-table-title a"
        )
        self.assertEqual(len(editable_templates), 1)
        # Enter copied template and modify title
        editable_templates[0].click()
        title_field = self.driver.find_element(By.CSS_SELECTOR, "input.title")
        title_field.send_keys(" COPY")
        import_id_field = self.driver.find_element(
            By.CSS_SELECTOR, "input.import-id"
        )
        import_id_field.send_keys("-1")
        self.driver.find_element(By.CSS_SELECTOR, "button.save").click()
        alert_element = WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-info"))
        )
        self.assertEqual(alert_element.is_displayed(), True)
        self.driver.refresh()
        # Download the file
        self.driver.find_element(By.CSS_SELECTOR, "button.download").click()
        alert_element = WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-info"))
        )
        self.assertEqual(alert_element.is_displayed(), True)
        time.sleep(1)
        file_path = os.path.join(
            self.download_dir, "copy-of-standard-article-copy.fidustemplate"
        )
        self.wait_until_file_exists(file_path, self.wait_time)
        assert os.path.isfile(file_path)
        self.driver.refresh()
        self.driver.find_element(By.CSS_SELECTOR, "button.close").click()
        time.sleep(1)
        upload_button = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.XPATH,
                    '//*[normalize-space()="Upload FIDUSTEMPLATE file"]',
                )
            )
        )
        upload_button.click()
        self.driver.find_element(
            By.CSS_SELECTOR, "#fidus-template-uploader"
        ).send_keys(file_path)
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Import"]'
        ).click()
        time.sleep(1)
        editable_templates = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr .fw-data-table-title a"
        )
        self.assertEqual(len(editable_templates), 2)
        os.remove(file_path)
        editable_templates[1].click()
        export_template_buttons = self.driver.find_elements(
            By.CSS_SELECTOR, ".export-templates button"
        )
        self.assertEqual(len(export_template_buttons), 3)
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Close"]'
        ).click()
        # Create file based on copied template
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Documents"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "li.new_document .dropdown"
        ).click()
        self.driver.find_elements(By.CSS_SELECTOR, ".fw-pulldown-item")[
            1
        ].click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.CLASS_NAME, "editor-toolbar"))
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "Article"
        )
        time.sleep(1)
        # Export full
        self.driver.find_element(
            By.CSS_SELECTOR, '.header-nav-item[title="File handling"]'
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Download"]'
        ).click()
        path = os.path.join(self.download_dir, "article.fidus")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        fat_file_path = os.path.join(self.download_dir, "fat.fidus")
        os.rename(path, fat_file_path)
        # Export slim
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Slim FIDUS"]'
        ).click()
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        slim_file_path = os.path.join(self.download_dir, "slim.fidus")
        os.rename(path, slim_file_path)
        # Exit editor
        self.driver.find_element(By.ID, "close-document-top").click()
        # Delete document
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".delete-document"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete"]'
        ).click()
        alert_element = WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-success"))
        )
        self.assertEqual(alert_element.is_displayed(), True)
        # Delete templates
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Templates"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".delete-doc-template"
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete"]'
        ).click()
        time.sleep(1)
        self.driver.find_element(
            By.CSS_SELECTOR, ".delete-doc-template"
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete"]'
        ).click()
        time.sleep(1)
        # Import slim document
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Documents"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title='Upload FIDUS document']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "#fidus-uploader").send_keys(
            slim_file_path
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        # Delete document
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".delete-document"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete"]'
        ).click()
        alert_element = WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-success"))
        )
        self.assertEqual(alert_element.is_displayed(), True)
        # Check that document template has no export templates
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Templates"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".fw-data-table-title"
        ).click()
        export_template_buttons = self.driver.find_elements(
            By.CSS_SELECTOR, ".export-templates button"
        )
        self.assertEqual(len(export_template_buttons), 1)
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Close"]'
        ).click()
        # Delete template
        self.driver.find_element(
            By.CSS_SELECTOR, ".delete-doc-template"
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete"]'
        ).click()
        # Import fat file
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Documents"]'
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title='Upload FIDUS document']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "#fidus-uploader").send_keys(
            fat_file_path
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        # Import slim file
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title='Upload FIDUS document']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "#fidus-uploader").send_keys(
            slim_file_path
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        time.sleep(1)
        # Check number of documents
        delete_links = self.driver.find_elements(
            By.CSS_SELECTOR, ".delete-document"
        )
        self.assertEqual(len(delete_links), 2)
        # Check number of templates
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Templates"]'
        ).click()
        template_links = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-data-table-title a"
        )
        self.assertEqual(len(template_links), 1)
        # Check export templates in template
        template_links[0].click()
        export_template_buttons = self.driver.find_elements(
            By.CSS_SELECTOR, ".export-templates button"
        )
        self.assertEqual(len(export_template_buttons), 3)
        os.remove(slim_file_path)
        os.remove(fat_file_path)



from base.management import BaseCommand
from django.core.management import call_command


class Command(BaseCommand):
    help = "Lint all source files"

    def add_arguments(self, parser):
        parser.add_argument(
            "--fix",
            action="store_true",
            dest="fix",
            default=False,
            help="Whether to also attempt to fix files",
        )

    def handle(self, *args, **options):
        fix = options["fix"]
        call_command("lint_js", fix=fix)
        call_command("lint_css", fix=fix)
        call_command("lint_py", fix=fix)



import os
import json
from copy import deepcopy

from prosemirror.model import Node, Schema
from prosemirror.transform import Step

from django.conf import settings

schema_json_path = os.path.join(
    settings.PROJECT_PATH, "static-libs/json/schema.json"
)

if os.path.exists(schema_json_path):
    with open(schema_json_path) as schema_file:
        schema = Schema(json.loads(schema_file.read()))
else:
    schema = None


def from_json(json):
    return Node.from_json(schema, json)


def apply(steps, node):
    for step_obj in steps:
        step = Step.from_json(schema, step_obj)
        step_result = step.apply(node)
        if step_result.ok:
            node = step_result.doc
        else:
            return False
    return node


def to_mini_json(node):
    # Similar to the ProseMirror internal toJSON function,
    # but leaving out attributes that have default values and dealing with
    # attributes that are objects.
    # Adapted from https://github.com/ProseMirror/prosemirror-model/blob/
    # 6d970507cd0da48653d3b72f2731a71a144a364b/src/node.js#L340-L351
    obj = {"type": node.type.name}

    for attr in node.attrs:
        if node.type.attrs[attr].default != node.attrs[attr]:
            if "attrs" not in obj:
                obj["attrs"] = {}
            obj["attrs"][attr] = deepcopy(node.attrs[attr])
    if getattr(node.content, "size", None):
        obj["content"] = list(map(to_mini_json, node.content.content))
    if len(node.marks):
        obj["marks"] = list(map(to_mini_mark_json, node.marks))
    if hasattr(node, "text"):
        obj["text"] = node.text
    return obj


def to_mini_mark_json(mark):
    # Adapted from https://github.com/ProseMirror/prosemirror-model/blob/
    # 6d970507cd0da48653d3b72f2731a71a144a364b/src/mark.js#L76-L83
    obj = {"type": mark.type.name}
    for attr in mark.attrs:
        if mark.type.attrs[attr].default != mark.attrs[attr]:
            if "attrs" not in obj:
                obj["attrs"] = {}
            obj["attrs"][attr] = deepcopy(mark.attrs[attr])
    return obj



from base.management import BaseCommand
import zipfile
import os
import shutil
import magic

from django.conf import settings


def zip_folder(path, zip_file):
    file_paths = []
    for root, dirs, files in os.walk(path):
        for file in files:
            out_file = os.path.join(root, file)
            relative_out_file = os.path.relpath(out_file, path)
            zip_file.write(out_file, relative_out_file)
            mimetype = magic.from_file(out_file, mime=True)
            # Override mimetype for CSS files
            if out_file.split(".")[-1].lower() == "css":
                mimetype = "text/css"
            file_paths.append(
                {"path": relative_out_file, "mimetype": mimetype}
            )
    return file_paths


def opf_entries(file_paths):
    opf_text = (
        "// This file is auto-generated. CHANGES WILL BE OVERWRITTEN! "
        "Re-generate by running ./manage.py bundle_mathlive.\n"
    )
    opf_text += "export const mathliveOpfIncludes = `\n"
    for index, file_path in enumerate(file_paths):
        mimetype = file_path["mimetype"]
        path = file_path["path"]
        if path.split(".")[-1] == "woff":
            mimetype = "font/woff"
        elif path.split(".")[-1] == "woff2":
            mimetype = "font/woff2"
        opf_text += (
            '<item id="mathlive-%d" href="css/%s" media-type="%s" />\n'
            % (
                index,
                path,
                mimetype,
            )
        )
    opf_text += "`"
    return opf_text


class Command(BaseCommand):
    args = ""
    help = (
        "Create a zip file containing the mathlive style files to be bundled "
        "with EPUB and HTML exports"
    )

    def handle(self, *args, **options):
        self.stdout.write("Bundling MathLive")
        # Copy MathLive CSS
        mathlive_css_path = os.path.join(
            settings.PROJECT_PATH, "static-libs/css/libs/mathlive/"
        )
        if not os.path.exists(mathlive_css_path):
            os.makedirs(mathlive_css_path)
        with open(
            os.path.join(
                settings.PROJECT_PATH,
                ".transpile/node_modules/mathlive/dist/mathlive-static.css",
            )
        ) as f:
            with open(
                os.path.join(mathlive_css_path, "mathlive.css"), "w"
            ) as g:
                g.write(f.read().replace("url(fonts/", "url(media/"))
        mathlive_fonts_path = os.path.join(
            settings.PROJECT_PATH, "static-libs/css/libs/mathlive/media/"
        )
        if not os.path.exists(mathlive_fonts_path):
            os.makedirs(mathlive_fonts_path)
        fontfiles_src_path = os.path.join(
            settings.PROJECT_PATH,
            ".transpile/node_modules/mathlive/dist/fonts/",
        )
        fontfiles = os.listdir(fontfiles_src_path)
        for filename in fontfiles:
            if filename[0] == ".":
                continue
            full_filename = os.path.join(fontfiles_src_path, filename)
            if os.path.isfile(full_filename):
                shutil.copy(full_filename, mathlive_fonts_path)
        zip_file_path = os.path.join(
            settings.PROJECT_PATH, "static-libs/zip/mathlive_style.zip"
        )
        zip_dir = os.path.dirname(zip_file_path)
        if not os.path.exists(zip_dir):
            os.makedirs(zip_dir)
        if os.path.exists(zip_file_path):
            os.remove(zip_file_path)
        zip_file = zipfile.ZipFile(zip_file_path, "w", zipfile.ZIP_DEFLATED)
        in_dir = os.path.dirname(mathlive_css_path)
        file_paths = zip_folder(in_dir, zip_file)
        zip_file.close()

        opf_file_contents = opf_entries(file_paths)
        opf_file_path = os.path.join(
            settings.PROJECT_PATH,
            "static-libs/js/modules/mathlive/opf_includes.js",
        )
        opf_dir = os.path.dirname(opf_file_path)
        if not os.path.exists(opf_dir):
            os.makedirs(opf_dir)
        with open(opf_file_path, "wb") as opf_file:
            opf_file.write(opf_file_contents.encode("utf8"))



# Generated by Django 3.1.4 on 2021-01-05 12:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("user", "0001_squashed_0003_auto_20151226_1110"),
    ]

    operations = [
        migrations.CreateModel(
            name="LoginAs",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "permissions": (
                    ("can_login_as", "Can login as another user"),
                ),
                "managed": False,
                "default_permissions": (),
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-03 10:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="publication",
            name="message_to_editor",
        ),
        migrations.AddField(
            model_name="publication",
            name="messages",
            field=models.JSONField(default=list),
        ),
    ]



import os
import time
from tempfile import mkdtemp

from testing.testcases import LiveTornadoTestCase
from testing.selenium_helper import SeleniumHelper
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait, Select
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from django.conf import settings


class ExportTest(LiveTornadoTestCase, SeleniumHelper):
    """Test whether Fidus Writer exports files in all the formats.
    Note that it does not validate the export files."""

    fixtures = [
        "initial_documenttemplates.json",
        "initial_styles.json",
    ]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.base_url = cls.live_server_url
        cls.download_dir = mkdtemp()
        driver_data = cls.get_drivers(1, cls.download_dir)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.driver.implicitly_wait(driver_data["wait_time"])
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        os.rmdir(cls.download_dir)
        super().tearDownClass()

    def setUp(self):
        self.verificationErrors = []
        self.accept_next_alert = True
        self.user1 = self.create_user(
            username="Yeti", email="yeti@snowman.com", passtext="otter"
        )

    def test_export(self):
        self.driver.get(self.base_url)
        self.driver.find_element(By.ID, "id-login").send_keys("Yeti")
        self.driver.find_element(By.ID, "id-password").send_keys("otter")
        self.driver.find_element(By.ID, "login-submit").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".new_document button")
            )
        ).click()
        # Set copyright
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (
                    By.CSS_SELECTOR,
                    "#header-navigation > div:nth-child(3) > span",
                )
            )
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "#header-navigation > div:nth-child(3) > "
                "div > ul > li:nth-child(6) > span"
            ),
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".holder").send_keys(
            "The Big Company"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".year").send_keys("2003")
        self.driver.find_element(By.CSS_SELECTOR, ".license").click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".license > option:nth-child(4)"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".license-start").send_keys(
            "2001-04-06"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "[aria-describedby=configure-copyright] button.fw-dark",
        ).click()

        self.driver.find_element(By.CSS_SELECTOR, ".article-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-title").send_keys(
            "Title"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "No styling"
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title=Strong]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "strong"
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title=Strong]"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "emph"
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title=Emphasis]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.ENTER
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fa-list-ol").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "ordered list"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.ENTER
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fa-list-ul").click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "bullet list"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.ENTER
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.ENTER
        )
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title=Blockquote]"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            "block quote"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.ENTER
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fa-link").click()
        self.driver.find_element(By.CSS_SELECTOR, ".link-title").click()
        self.driver.find_element(By.CSS_SELECTOR, ".link-title").send_keys(
            "Sports"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".link").click()
        self.driver.find_element(By.CSS_SELECTOR, ".link").send_keys(
            "https://www.sports.com"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        self.driver.find_element(By.CSS_SELECTOR, ".fa-asterisk").click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".footnote-container > p"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".footnote-container > p"
        ).send_keys("A footnote")
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").click()
        self.driver.find_element(By.CSS_SELECTOR, ".fa-book").click()
        # click on 'Register new source' button
        register_new_source = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CLASS_NAME, "register-new-bib-source")
            )
        )
        register_new_source.click()

        # select source
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located((By.ID, "select-bibtype"))
        )

        # click on article
        Select(
            self.driver.find_element(By.ID, "select-bibtype")
        ).select_by_visible_text("Article")

        # fill the values
        title_of_publication = self.driver.find_element(
            By.CSS_SELECTOR, ".journaltitle .ProseMirror"
        )
        title_of_publication.click()
        title_of_publication.send_keys("My publication title")

        title = self.driver.find_element(
            By.CSS_SELECTOR, ".title .ProseMirror"
        )
        title.click()
        title.send_keys("My title")

        author_firstName = self.driver.find_element(
            By.CSS_SELECTOR, ".author .given .ProseMirror"
        )
        author_firstName.click()
        author_firstName.send_keys("John")

        author_lastName = self.driver.find_element(
            By.CSS_SELECTOR, ".family .ProseMirror"
        )
        author_lastName.click()
        author_lastName.send_keys("Doe")

        publication_date = self.driver.find_element(
            By.CSS_SELECTOR, ".date .date"
        )
        publication_date.click()
        publication_date.send_keys("2012")

        # click on Submit button
        self.driver.find_element(
            By.XPATH,
            '//*[contains(@class, "ui-button") and normalize-space()="Submit"]',
        ).click()

        # Wait for source to be listed
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (
                    By.CSS_SELECTOR,
                    "#selected-cite-source-table .selected-source",
                )
            )
        )
        # click on Insert button
        self.driver.find_element(By.CSS_SELECTOR, ".insert-citation").click()
        self.driver.find_element(
            By.CSS_SELECTOR, 'button[title="Horizontal line"]'
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".article-body").send_keys(
            Keys.DOWN
        )
        # Insert figure
        button = self.driver.find_element(By.XPATH, '//*[@title="Figure"]')
        button.click()

        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "#figure-dialog span.math-field")
            )
        )

        time.sleep(1)  # Needed to ensure next lines work

        # click on 'Insert image' button
        WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[normalize-space()="Insert image"]')
            )
        ).click()

        upload_button = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[normalize-space()="Add new image"]')
            )
        )

        upload_button.click()

        # image path
        image_path = os.path.join(
            settings.PROJECT_PATH, "document/tests/uploads/image.png"
        )

        # in order to select the image we send the image path in the
        # LOCAL MACHINE to the input tag
        upload_image_url = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.XPATH, '//*[@id="editimage"]/div[1]/input[2]')
            )
        )
        upload_image_url.send_keys(image_path)

        # click on 'Upload' button
        self.driver.find_element(
            By.XPATH,
            '//*[contains(@class, "ui-button") and normalize-space()="Upload"]',
        ).click()

        # click on 'Use image' button
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, ".fw-data-table i.fa-check")
            )
        )

        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Use image"]'
        ).click()

        # Set copyright info
        self.driver.find_element(
            By.CSS_SELECTOR, "div.figure-preview > div > span > i"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            (
                "body > div.ui-content-menu.ui-corner-all.ui-widget."
                "ui-widget-content.ui-front > div > div > ul > li:nth-child(1)"
            ),
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".holder").send_keys(
            "Johannes Wilm"
        )
        self.driver.find_element(By.CSS_SELECTOR, ".year").send_keys("1998")
        self.driver.find_element(By.CSS_SELECTOR, ".license").click()
        self.driver.find_element(
            By.CSS_SELECTOR, ".license > option:nth-child(2)"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".license-start").send_keys(
            "1998-04-23"
        )
        self.driver.find_element(
            By.CSS_SELECTOR,
            "[aria-describedby=configure-copyright] button.fw-dark",
        ).click()

        # click on 'Insert' button
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()

        caption = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "div.article-body figure figcaption")
            )
        )
        caption.click()
        caption.send_keys("Figure")

        ActionChains(self.driver).send_keys(Keys.RIGHT).perform()
        # Add table
        self.driver.find_element(By.CSS_SELECTOR, ".fa-table").click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "table.insert-table-selection tr:nth-child(2) > td:nth-child(2)",
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td:nth-child(1) > p"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td:nth-child(1) > p"
        ).send_keys("one")
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td:nth-child(2) > p"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td:nth-child(2) > p"
        ).send_keys("two")
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(2) > td:nth-child(1) > p"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(2) > td:nth-child(1) > p"
        ).send_keys("three")
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(2) > td:nth-child(2)"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(2) > td:nth-child(2)"
        ).send_keys("four")
        # Add table caption
        self.driver.find_element(
            By.CSS_SELECTOR, "div.table-100 > button"
        ).click()
        self.driver.find_element(
            By.CSS_SELECTOR,
            "body > div.ui-content-menu > div > div > ul > li:nth-child(16)",
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "div.table-category").click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Table"]'
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "div.table-caption").click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Enable"]'
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        caption = WebDriverWait(self.driver, self.wait_time).until(
            EC.presence_of_element_located(
                (By.CSS_SELECTOR, "div.article-body table caption")
            )
        )

        caption.click()

        caption.send_keys("Table Caption")
        # Document with many features has been created let's see if we can
        # export it from the editor.

        # HTML (old)
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="HTML (old)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.html.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # HTML (new)
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="HTML (new)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.html.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # EPUB
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Epub"]'
        ).click()
        path = os.path.join(self.download_dir, "title.epub")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # LaTeX
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="LaTeX"]'
        ).click()
        path = os.path.join(self.download_dir, "title.latex.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # JATS
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="JATS"]'
        ).click()
        path = os.path.join(self.download_dir, "title.jats.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # DOCX
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Classic (DOCX)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.docx")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # ODT
        self.driver.find_element(
            By.CSS_SELECTOR,
            '.header-nav-item[title="Export of the document contents"]',
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Free (ODT)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.odt")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # Save a revision
        self.driver.find_element(
            By.CSS_SELECTOR, '.header-nav-item[title="File handling"]'
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Save revision"]'
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".revision-note").send_keys(
            "First revision"
        )
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Save"]'
        ).click()
        # Wait for revision save to be done
        WebDriverWait(self.driver, self.wait_time).until(
            EC.visibility_of_element_located((By.CLASS_NAME, "alerts-success"))
        )
        # Exit the editor
        self.driver.find_element(By.ID, "close-document-top").click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.ID, "preferences-btn"))
        )

        # Export from overview
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td > label"
        ).click()

        # Native
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as FIDUS"]'
        ).click()
        path = os.path.join(self.download_dir, "title.fidus")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # Slim native
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as Slim FIDUS"]'
        ).click()
        path = os.path.join(self.download_dir, "title.fidus")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        # We keep the file for the upload test below
        upload_slim_path = os.path.join(self.download_dir, "upload_slim.fidus")
        os.rename(path, upload_slim_path)

        # EPUB
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as Epub"]'
        ).click()
        path = os.path.join(self.download_dir, "title.epub")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # HTML (old)
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as HTML (old)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.html.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # HTML (new)
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as HTML (new)"]'
        ).click()
        path = os.path.join(self.download_dir, "title.html.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # LaTeX
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as LaTeX"]'
        ).click()
        path = os.path.join(self.download_dir, "title.latex.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # JATS
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Export selected as JATS"]'
        ).click()
        path = os.path.join(self.download_dir, "title.jats.zip")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        os.remove(path)

        # Revision
        self.driver.find_element(By.CSS_SELECTOR, ".revisions").click()
        self.driver.find_element(By.CSS_SELECTOR, ".download-revision").click()
        path = os.path.join(self.download_dir, "title.fidus")
        self.wait_until_file_exists(path, self.wait_time)
        assert os.path.isfile(path)
        upload_full_path = os.path.join(self.download_dir, "upload_full.fidus")
        os.rename(path, upload_full_path)
        # We keep the file to test import below
        self.driver.find_element(By.CSS_SELECTOR, ".recreate-revision").click()
        self.assertEqual(
            len(
                self.driver.find_elements(
                    By.CSS_SELECTOR, "#revisions-dialog > table > tbody tr"
                )
            ),
            1,
        )
        self.driver.find_element(By.CSS_SELECTOR, ".delete-revision").click()
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        self.assertEqual(
            len(
                self.driver.find_elements(
                    By.CSS_SELECTOR, "#revisions-dialog > table > tbody tr"
                )
            ),
            0,
        )
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-orange").click()

        # Delete document
        documents = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr a.fw-data-table-title"
        )
        self.assertEqual(len(documents), 2)
        self.driver.find_element(
            By.CSS_SELECTOR, "tr:nth-child(1) > td > label"
        ).click()
        WebDriverWait(self.driver, self.wait_time).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, ".dt-bulk-dropdown"))
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Delete selected"]'
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        documents = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr a.fw-data-table-title"
        )
        self.assertEqual(len(documents), 1)
        self.driver.find_element(
            By.CSS_SELECTOR, ".fw-contents tbody tr .delete-document"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "button.fw-dark").click()
        time.sleep(1)
        documents = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr a.fw-data-table-title"
        )
        self.assertEqual(len(documents), 0)
        # We delete the image so our import will cause an image import
        self.driver.find_element(
            By.CSS_SELECTOR, "a[href='/usermedia/']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, ".delete-image").click()
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        # We import the fidus file
        self.driver.find_element(By.CSS_SELECTOR, "a[href='/']").click()
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title='Upload FIDUS document']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "#fidus-uploader").send_keys(
            upload_full_path
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        documents = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr a.fw-data-table-title"
        )
        self.assertEqual(len(documents), 1)
        # We delete our downloaded fidus file
        os.remove(upload_full_path)
        # Upload slim file
        self.driver.find_element(
            By.CSS_SELECTOR, "button[title='Upload FIDUS document']"
        ).click()
        self.driver.find_element(By.CSS_SELECTOR, "#fidus-uploader").send_keys(
            upload_slim_path
        )
        self.driver.find_element(By.CSS_SELECTOR, ".fw-dark").click()
        time.sleep(1)
        documents = self.driver.find_elements(
            By.CSS_SELECTOR, ".fw-contents tbody tr a.fw-data-table-title"
        )
        self.assertEqual(len(documents), 2)
        # We delete our downloaded fidus file
        os.remove(upload_slim_path)



import json

from tornado.httpclient import AsyncHTTPClient, HTTPRequest, HTTPError
from allauth.socialaccount.models import SocialToken
from allauth.socialaccount.providers.gitlab.views import GitLabOAuth2Adapter

GITLAB_BASE_URL = f"{GitLabOAuth2Adapter.provider_base_url}/api/v4/"

ALLOWED_METHODS = ["GET", "POST", "PATCH"]


def get_headers(token):
    return {
        "Authorization": f"Bearer {token}",
        "User-Agent": "Fidus Writer",
        "Content-Type": "application/json",
    }


async def prepare(proxy_connector, path_parts, user):
    path = "/".join(path_parts)
    path_part = path_parts.pop(0) if len(path_parts) else None
    social_token = SocialToken.objects.filter(
        account__user=user, account__provider="gitlab"
    ).first()
    if not social_token:
        proxy_connector.set_status(401)
        return
    headers = get_headers(social_token.token)
    if path_part == "repo":
        await get_repo(proxy_connector, path_parts, user, headers)
        return
    method = proxy_connector.request.method
    query = proxy_connector.request.query
    url = f"{GITLAB_BASE_URL}{path}"
    if query:
        url += "?" + query
    if method == "GET":
        body = None
    else:
        body = proxy_connector.request.body
    request = HTTPRequest(url, method, headers, body=body, request_timeout=120)
    http = AsyncHTTPClient()
    try:
        response = await http.fetch(request)
    except HTTPError as e:
        if e.response.code == 404:
            # We remove the 404 response so it will not show up as an
            # error in the browser
            proxy_connector.write("[]")
        else:
            proxy_connector.set_status(e.response.code)
            proxy_connector.write(e.response.body)
    except Exception as e:
        proxy_connector.set_status(500)
        proxy_connector.write("Error: %s" % e)
    else:
        proxy_connector.set_status(response.code)
        proxy_connector.write(response.body)


async def get_repo(proxy_connector, path_parts, user, headers):
    id = path_parts.pop(0) if len(path_parts) else None
    files = []
    next_url = (
        f"{GITLAB_BASE_URL}projects/{id}/repository/tree"
        "?recursive=true&per_page=4&pagination=keyset"
    )
    while next_url:
        request = HTTPRequest(next_url, "GET", headers, request_timeout=120)
        http = AsyncHTTPClient()
        try:
            response = await http.fetch(request)
        except HTTPError as e:
            proxy_connector.set_status(e.response.code)
            proxy_connector.write(e.response.body)
            return
        except Exception as e:
            proxy_connector.set_status(500)
            proxy_connector.write("Error: %s" % e)
            return
        files += json.loads(response.body)
        next_url = False
        for link_info in response.headers["Link"].split(", "):
            link, rel = link_info.split("; ")
            if rel == 'rel="next"':
                next_url = link[1:-1]
    proxy_connector.write(json.dumps(files))


def gitlabrepo2repodata(gitlab_repo):
    return {
        "type": "gitlab",
        "name": gitlab_repo["path_with_namespace"],
        "id": gitlab_repo["id"],
        "branch": gitlab_repo["default_branch"],
    }


async def get_repos(proxy_connector, gitlab_token):
    # TODO: API documentation unclear on whether pagination is required.
    headers = get_headers(gitlab_token)
    repos = []
    url = f"{GITLAB_BASE_URL}projects?min_access_level=30&simple=true"
    request = HTTPRequest(url, "GET", headers, request_timeout=120)
    http = AsyncHTTPClient()
    try:
        response = await http.fetch(request)
    except HTTPError as e:
        if e.response.code == 404:
            # We remove the 404 response so it will not show up as an
            # error in the browser
            return []
        else:
            proxy_connector.set_status(e.response.code)
            proxy_connector.write(e.response.body)
        return []
    except Exception as e:
        proxy_connector.set_status(500)
        proxy_connector.write("Error: %s" % e)
        return []
    content = json.loads(response.body)
    repos += map(gitlabrepo2repodata, content)
    return repos



# Generated by Django 3.1.4 on 2021-04-05 16:21

from django.db import migrations


def change_access_right_to_generic_key(apps, schema_editor):
    AccessRight = apps.get_model("document", "AccessRight")
    ContentType = apps.get_model("contenttypes", "ContentType")
    user_ct = ContentType.objects.filter(
        app_label="user", model="user"
    ).first()
    access_rights = AccessRight.objects.all()
    for access_right in access_rights:
        access_right.holder_id = access_right.user_id
        access_right.holder_type = user_ct
        access_right.save()


def reverse_change_access_right_to_generic_key(apps, schema_editor):
    AccessRight = apps.get_model("document", "AccessRight")
    ContentType = apps.get_model("contenttypes", "ContentType")
    access_rights = AccessRight.objects.all()
    user_ct = ContentType.objects.filter(
        app_label="user", model="user"
    ).first()
    for access_right in access_rights:
        if access_right.holder_type != user_ct:
            # This is not a regular user. Don't try to revert
            continue
        access_right.user_id = access_right.holder_id
        access_right.save()


class Migration(migrations.Migration):

    dependencies = [
        ("document", "0011_auto_20210405_1820"),
        ("user", "0004_auto_20210402_2049"),
    ]

    operations = [
        migrations.RunPython(
            change_access_right_to_generic_key,
            reverse_change_access_right_to_generic_key,
        ),
    ]



from tornado.web import RequestHandler
from base.django_handler_mixin import DjangoHandlerMixin

from . import common_proxy
from . import github_proxy
from . import gitlab_proxy


class Proxy(DjangoHandlerMixin, RequestHandler):
    SUPPORTED_METHODS = ["GET", "POST", "PATCH"]

    async def prepare(self):
        user = self.get_current_user()
        path_parts = self.path_args[0].split("/")
        path_part = path_parts.pop(0) if len(path_parts) else None
        if not user.is_authenticated:
            self.set_status(401)
        elif path_part == "all":
            await common_proxy.prepare(self, path_parts, user)
        elif path_part == "github":
            await github_proxy.prepare(self, path_parts, user)
        elif path_part == "gitlab":
            await gitlab_proxy.prepare(self, path_parts, user)
        else:
            self.set_status(404)
        self.finish()



import time
import logging
from testing.testcases import LiveTornadoTestCase
from .editor_helper import EditorHelper
from document.ws_views import WebSocket
from document import prosemirror
from selenium.webdriver.common.by import By


class SimpleMessageExchangeTests(LiveTornadoTestCase, EditorHelper):
    """
    Tests in which one user works on the document and simulates
    loss of socket messages.
    """

    user = None
    TEST_TEXT = "Lorem ipsum dolor sit amet."
    fixtures = [
        "initial_documenttemplates.json",
        "initial_styles.json",
    ]

    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        driver_data = cls.get_drivers(1)
        cls.driver = driver_data["drivers"][0]
        cls.client = driver_data["clients"][0]
        cls.wait_time = driver_data["wait_time"]

    @classmethod
    def tearDownClass(cls):
        cls.driver.quit()
        super().tearDownClass()

    def setUp(self):
        self.user = self.create_user()
        self.login_user(self.user, self.driver, self.client)
        self.doc = self.create_new_document()

    def test_client_losing_server_messages(self):
        """
        Test one client trying to edit document while online.
        But the messages supposed to be received from the server
        are lost during transmission.
        (The messages loss is simulated manually)
        """
        self.load_document_editor(self.driver, self.doc)

        self.add_title(self.driver)
        self.driver.find_element(By.CLASS_NAME, "article-body").click()

        # Type lots of text to increment the server message count.
        socket_object = WebSocket.sessions[self.doc.id]["participants"][0]
        self.type_text(self.driver, self.TEST_TEXT)
        self.type_text(self.driver, self.TEST_TEXT)
        self.type_text(self.driver, self.TEST_TEXT)
        self.type_text(self.driver, self.TEST_TEXT)

        # Assert that the server count changed in client
        old_server_count = self.driver.execute_script(
            "return window.theApp.page.ws.messages.server"
        )
        # We simulate lost messages by changing the counter manually.
        self.driver.execute_script("window.theApp.page.ws.messages.server = 1")

        # Assert that the server count changed in client
        server_count = self.driver.execute_script(
            "return window.theApp.page.ws.messages.server"
        )

        self.assertNotEqual(old_server_count, server_count)

        self.assertEqual(server_count, 1)

        # Now try typing text again to see if server goes into loop.
        self.type_text(self.driver, self.TEST_TEXT)

        # Check the Socket object to verify that server isn't going in a loop.
        doc_message_count = 0
        doc_data = None
        for message in socket_object.messages["last_ten"]:
            if "type" in message.keys():
                if message["type"] == "doc_data":
                    doc_message_count += 1
                    doc_data = message["doc"]["content"]

        # We should've only sent the doc_data message once .
        self.assertEqual(doc_message_count, 1)

        # Now assert that the document reloaded in front end too !
        doc_content = prosemirror.to_mini_json(
            prosemirror.from_json(
                self.driver.execute_script(
                    "return window.theApp.page.docInfo."
                    "confirmedDoc.firstChild.toJSON()"
                )
            )
        )

        self.assertEqual(doc_data, doc_content)

    def test_server_receives_failing_patch(self):
        """
        The server receives a patch from the server that is failing. It should
        be stopped at the server and not be applied. If the patch has one part
        that is valid and another that is invalid, none of them should be
        applied.
        """
        self.load_document_editor(self.driver, self.doc)

        self.add_title(self.driver)
        self.driver.find_element(
            By.CSS_SELECTOR,
            "#header-navigation > div:nth-child(3) > span",  # Settings
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Text Language"]'
        ).click()
        self.driver.find_element(
            By.XPATH, '//*[normalize-space()="Spanish"]'
        ).click()
        socket_object = WebSocket.sessions[self.doc.id]["participants"][0]
        diff_script = (
            "theApp.page.ws.send(()=>({"
            "type: 'diff',"
            "v: theApp.page.docInfo.version,"
            "rid: theApp.page.mod.collab.doc.confirmStepsRequestCounter++,"
            "cid: theApp.page.client_id,"
            "jd: ["
            "{op: 'add', path: '/attrs/language', value: 'de-DE'},"  # valid
            "{op: 'remove', path: '/fish'}"  # invalid
            "]"
            "}))"
        )
        with self.settings(JSONPATCH=True):
            logging.disable(logging.CRITICAL)
            self.driver.execute_script(diff_script)
            time.sleep(1)
            logging.disable(logging.NOTSET)
        doc_data = False
        patch_error = 0
        for message in socket_object.messages["last_ten"]:
            if message["type"] == "doc_data":
                doc_data = message["doc"]["content"]
            elif message["type"] == "patch_error":
                patch_error += 1
        # The language should still be Spanish
        self.assertEqual(doc_data["attrs"]["language"], "es")
        # There should be one patch error
        self.assertEqual(patch_error, 1)
        system_message = self.driver.find_element(
            By.CSS_SELECTOR, "div.ui-dialog-content.ui-widget-content > p"
        )
        assert system_message.text == (
            "Your document was out of sync and has been reset."
        )



from django.urls import re_path

from . import views

urlpatterns = [
    re_path(
        "^get_doc_info/$",
        views.get_doc_info,
        name="website_get_doc_info",
    ),
    re_path("^submit_doc/$", views.submit_doc, name="website_submit_doc"),
    re_path("^reject_doc/$", views.reject_doc, name="website_reject_doc"),
    re_path("^review_doc/$", views.review_doc, name="website_review_doc"),
    re_path("^publish_doc/$", views.publish_doc, name="website_publish_doc"),
    re_path(
        "^list_publications/$",
        views.list_publications,
        name="website_list_publications",
    ),
    re_path(
        "^list_publications/(?P<per_page>[0-9]+)/(?P<page_number>[0-9]+)/$",
        views.list_publications,
        name="website_list_publications",
    ),
    re_path(
        "^get_publication/(?P<id>[0-9]+)/$",
        views.get_publication,
        name="website_get_publication",
    ),
    re_path(
        "^get_style/$",
        views.get_style,
        name="website_get_style",
    ),
]



from django.contrib.auth.decorators import login_required
from django.views.decorators.http import require_GET, require_POST
from django.http import JsonResponse, HttpResponseForbidden, HttpResponse
from base.decorators import ajax_required
from django.db.models import Q
from book.models import Book
from . import models


@login_required
@ajax_required
@require_GET
def get_book_repos(request):
    response = {}
    status = 200
    book_repos = models.BookRepository.objects.filter(
        Q(book__owner=request.user)
        | Q(
            book__bookaccessright__holder_id=request.user.id,
            book__bookaccessright__holder_type__model="user",
        )
    ).distinct()
    response["book_repos"] = {}
    for repo in book_repos:
        response["book_repos"][repo.book.id] = {
            "repo_id": repo.repo_id,
            "repo_name": repo.repo_name,
            "repo_type": repo.repo_type,
            "export_epub": repo.export_epub,
            "export_unpacked_epub": repo.export_unpacked_epub,
            "export_html": repo.export_html,
            "export_unified_html": repo.export_unified_html,
            "export_latex": repo.export_latex,
        }
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def update_book_repo(request):
    book_id = request.POST["book_id"]
    book = Book.objects.filter(id=book_id).first()
    if not book or (
        book.owner != request.user
        and not book.bookaccessright_set.filter(
            holder_id=request.user.id,
            holder_type__model="user",
            rights="write",
        ).exists()
    ):
        return HttpResponseForbidden()
    models.BookRepository.objects.filter(book_id=book_id).delete()
    repo_id = request.POST["repo_id"]
    if repo_id == 0:
        status = 200
    else:
        models.BookRepository.objects.create(
            book_id=book_id,
            repo_id=repo_id,
            repo_name=request.POST["repo_name"],
            repo_type=request.POST["repo_type"],
            export_epub=request.POST["export_epub"] == "true",
            export_unpacked_epub=request.POST["export_unpacked_epub"]
            == "true",
            export_html=request.POST["export_html"] == "true",
            export_unified_html=request.POST["export_unified_html"] == "true",
            export_latex=request.POST["export_latex"] == "true",
        )
        status = 201
    return HttpResponse(status=status)



import json

from django.http import JsonResponse, HttpRequest
from django.db.models import Q
from django.contrib.auth import logout, update_session_auth_hash
from django.contrib.auth.decorators import login_required
from django.conf import settings
from django.shortcuts import HttpResponseRedirect
from django.views.decorators.http import require_POST
from django.contrib.auth import get_user_model
from django.core.validators import validate_email
from django.core.exceptions import ValidationError
from django.core.files import File

from base.decorators import ajax_required
from .forms import UserForm
from document.models import AccessRight
from .models import UserInvite
from .helpers import Avatars
from . import emails

from allauth.account.models import (
    EmailAddress,
    EmailConfirmation,
    EmailConfirmationHMAC,
)
from allauth.account.views import SignupView
from allauth.account import signals
from allauth.socialaccount.models import SocialAccount
from allauth.socialaccount.signals import social_account_removed
from django.contrib.auth.forms import PasswordChangeForm
from allauth.account.forms import AddEmailForm

from avatar.models import Avatar
from avatar import views as avatarviews
from avatar.forms import UploadAvatarForm
from avatar.signals import avatar_updated


@login_required
@ajax_required
@require_POST
def password_change(request):
    """
    Change password
    """
    response = {}
    form = PasswordChangeForm(user=request.user, data=request.POST)
    if form.is_valid():
        status = 200
        form.save()
        # Updating the password logs out all other sessions for the user
        # except the current one.
        update_session_auth_hash(request, form.user)
    else:
        response["msg"] = form.errors
        status = 201

    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def add_email(request):
    """
    Add email address
    """
    response = {}
    add_email_form = AddEmailForm(request.user, request.POST)
    if add_email_form.is_valid():
        status = 200
        email_address = add_email_form.save(request)
        signals.email_added.send(
            sender=request.user.__class__,
            request=request,
            user=request.user,
            email_address=email_address,
        )
    else:
        status = 201
        response["msg"] = add_email_form.errors

    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete_email(request):
    response = {}
    email = request.POST["email"]
    response["msg"] = f"Removed e-mail address {email}"
    status = 200
    email_address = EmailAddress.objects.filter(
        user=request.user, email=email, primary=False
    ).first()
    if not email_address:
        return JsonResponse({"msg": "Cannot remove email."}, status=404)
    email_address.delete()
    signals.email_removed.send(
        sender=request.user.__class__,
        request=request,
        user=request.user,
        email_address=email_address,
    )
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def primary_email(request):
    response = {}
    email = request.POST["email"]
    email_address = EmailAddress.objects.filter(
        user=request.user, email=email, verified=True
    ).first()
    if not email_address:
        return JsonResponse({"msg": "Cannot set primary email."}, 404)
    from_email_address = EmailAddress.objects.filter(
        user=request.user, primary=True
    ).first()
    status = 200
    email_address.set_as_primary()
    response["msg"] = "Primary e-mail address set"
    signals.email_changed.send(
        sender=request.user.__class__,
        request=request,
        user=request.user,
        from_email_address=from_email_address,
        to_email_address=email_address,
    )
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete_socialaccount(request):
    account = SocialAccount.objects.filter(
        id=request.POST["socialaccount"], user=request.user
    ).first()
    if not account:
        return JsonResponse({"msg": "Unknown account"}, status=404)
    account.delete()
    social_account_removed.send(
        sender=SocialAccount, request=request, socialaccount=account
    )
    return JsonResponse({"msg": "Deleted account"}, status=200)


def create_avatar_thumbnail(avatar, size=settings.AVATAR_DEFAULT_SIZE):
    if not avatar.thumbnail_exists(size):
        avatar.create_thumbnail(size)
        # Now check if the thumbnail was actually created
        if not avatar.thumbnail_exists(size):
            # Thumbnail was not saved. There must be some PIL bug
            # with this image type. We store the original file instead.
            avatar.avatar.storage.save(
                avatar.avatar_name(size),
                File(avatar.avatar.storage.open(avatar.avatar.name, "rb")),
            )


@login_required
@ajax_required
@require_POST
def upload_avatar(request):
    """
    Upload avatar image
    """
    response = {}
    status = 405

    avatar, avatars = avatarviews._get_avatars(request.user)
    upload_avatar_form = UploadAvatarForm(
        None, request.FILES, user=request.user
    )
    if upload_avatar_form.is_valid():
        avatar = Avatar(
            user=request.user,
            primary=True,
        )
        image_file = request.FILES["avatar"]
        avatar.avatar.save(image_file.name, image_file)
        avatar.save()
        create_avatar_thumbnail(avatar)
        avatar_updated.send(sender=Avatar, user=request.user, avatar=avatar)
        response["avatar"] = avatar.avatar_url(settings.AVATAR_DEFAULT_SIZE)
        status = 200
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete_avatar(request):
    """
    Delete avatar image
    """
    response = {}
    status = 405
    avatar, avatars = avatarviews._get_avatars(request.user)
    if avatar is None:
        response["error"] = "User has no avatar"
    else:
        aid = avatar.id
        for a in avatars:
            # Find the next best avatar, and set it as the new primary
            if a.id != aid:
                a.primary = True
                a.save()
                avatar_updated.send(sender=Avatar, user=request.user, avatar=a)
                response["avatar"] = a.avatar_url(settings.AVATAR_DEFAULT_SIZE)
                break
        Avatar.objects.filter(pk=aid).delete()
        status = 200
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete_user(request):
    """
    Delete the user
    """
    response = {}
    user = request.user
    # Only remove users who are not marked as having staff status
    # to prevent administratoras from deleting themselves accidentally.
    if not user.check_password(request.POST["password"]):
        status = 401
    elif user.is_staff:
        status = 403
    else:
        logout(request)
        user.delete()
        status = 200
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def save_profile(request):
    """
    Save user profile information
    """
    response = {}
    form_data = json.loads(request.POST["form_data"])
    User = get_user_model()
    user_object = User.objects.get(pk=request.user.pk)
    user_form = UserForm(form_data["user"], instance=user_object)
    if user_form.is_valid():
        user_form.save()
        status = 200
    else:
        response["errors"] = user_form.errors
        status = 422

    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def list_contacts(request):
    response = {}
    status = 200
    response["contacts"] = []
    avatars = Avatars()
    for user in request.user.contacts.all():
        response["contacts"].append(
            {
                "id": user.id,
                "name": user.readable_name,
                "username": user.get_username(),
                "email": user.email,
                "avatar": avatars.get_url(user),
                "type": "user",
            }
        )
    for invite in request.user.invites_by.all():
        response["contacts"].append(
            {
                "id": invite.id,
                "name": invite.username,
                "username": invite.username,
                "email": invite.email,
                "avatar": None,
                "type": "userinvite",
            }
        )
    for invite in request.user.invites_to.select_related("by").all():
        response["contacts"].append(
            {
                "id": invite.id,
                "name": invite.by.readable_name,
                "username": invite.by.get_username(),
                "email": invite.by.email,
                "avatar": avatars.get_url(invite.by),
                "type": "to_userinvite",
            }
        )
    return JsonResponse(response, status=status)


def is_email(string):
    try:
        validate_email(string)
        return True
    except ValidationError:
        return False


@login_required
@ajax_required
@require_POST
def invites_add(request):
    """
    Add a userinvite as a contact of the current user
    """
    response = {}
    contact_user = False
    email = False
    errored = False
    status = 202
    user_string = request.POST["user_string"]
    if (
        UserInvite.objects.filter(username=user_string)
        .filter(by=request.user)
        .exists()
    ):
        # 'This person is already in your invites!'
        response["error"] = 2
        return JsonResponse(response, status=status)
    User = get_user_model()
    contact_user = User.objects.filter(username=user_string).first()
    if contact_user:
        email = contact_user.email
    elif is_email(user_string):
        email = user_string
        email_address = EmailAddress.objects.filter(email=user_string).first()
        if email_address:
            contact_user = email_address.user
    if contact_user:
        if contact_user.pk is request.user.pk:
            # 'You cannot add yourself to your contacts!'
            response["error"] = 1
            errored = True
        elif request.user.contacts.filter(id=contact_user.id).first():
            # 'This person is already in your contacts!'
            response["error"] = 2
            errored = True
    elif not email:
        # 'Invalid email!'
        response["error"] = 3
        errored = True
    if not errored:
        invite = UserInvite.objects.create(
            username=user_string,
            email=email,
            by=request.user,
            to=contact_user,
        )
        sender = request.user.readable_name
        email = invite.email
        link = HttpRequest.build_absolute_uri(
            request, invite.get_relative_url()
        )
        emails.send_invite_notification(sender, email, link)
        response["contact"] = {
            "id": invite.pk,
            "name": invite.username,
            "email": invite.email,
            "avatar": None,
            "type": "userinvite",
        }
        status = 201
    return JsonResponse(response, status=status)


def invites_connect(user, key=None):
    invites = UserInvite.objects.filter(to__isnull=True)
    if key:
        invites = invites.filter(
            key=key,
        )
    else:
        invites = invites.filter(
            Q(email__in=user.emailaddress_set.all()) | Q(email=user.email)
        )
    if len(invites) == 0:
        return False
    for invite in invites:
        invite.to = user
        invite.save()
    return True


@login_required
@ajax_required
@require_POST
def invite(request):
    response = {}
    status = 200
    key = request.POST["key"]
    connected = invites_connect(request.user, key)
    if connected:
        response["redirect"] = "/user/contacts/"
    else:
        response["redirect"] = "/"
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def invites_accept(request):
    response = {}
    status = 200
    invites = json.loads(request.POST["invites"])
    response["contacts"] = []
    avatars = Avatars()
    for invite in invites:
        ui = (
            UserInvite.objects.filter(id=invite["id"], to=request.user)
            .select_related("by")
            .first()
        )
        if ui:
            response["contacts"].append(
                {
                    "id": ui.by.id,
                    "name": ui.by.readable_name,
                    "email": ui.by.email,
                    "avatar": avatars.get_url(ui.by),
                    "type": "user",
                }
            )
            link = HttpRequest.build_absolute_uri(request, "/user/contacts/")
            emails.send_accept_notification(
                ui.by.readable_name, ui.by.email, ui.to.readable_name, link
            )
            ui.apply()
    return JsonResponse(response, status=status)


def invite_decline(ui, link):
    # Remove this invite. All connected access rights will be deleted
    # automatically.
    emails.send_decline_notification(
        ui.by.readable_name, ui.by.email, ui.to.readable_name, link
    )
    ui.delete()


@login_required
@ajax_required
@require_POST
def invites_decline(request):
    """
    Decline an invite
    """
    response = {}
    invites = json.loads(request.POST["invites"])
    for invite in invites:
        for ui in request.user.invites_to.filter(id=invite["id"]):
            link = HttpRequest.build_absolute_uri(request, "/user/contacts/")
            invite_decline(ui, link)
    status = 200
    return JsonResponse(response, status=status)


@login_required
@ajax_required
@require_POST
def delete_contacts(request):
    """
    Delete a contact
    """
    response = {}
    former_contacts = json.loads(request.POST["contacts"])
    for former_contact in former_contacts:
        if former_contact["type"] == "user":
            # Revoke all permissions given to this user
            AccessRight.objects.filter(
                user__id=former_contact["id"], document__owner=request.user
            ).delete()
            # Revoke all permissions received from this user
            AccessRight.objects.filter(
                user=request.user, document__owner_id=former_contact["id"]
            ).delete()
            # Remove the user from the contacts
            request.user.contacts.remove(
                request.user.contacts.filter(id=former_contact["id"]).first()
            )
        elif former_contact["type"] == "userinvite":
            # Delete the userinvite. All connected access rights will be
            # deleted automatically.
            request.user.invites_by.filter(id=former_contact["id"]).delete()
        elif former_contact["type"] == "to_userinvite":
            # Remove this invite. All connected access rights will be deleted
            # automatically.
            for ui in request.user.invites_to.filter(id=former_contact["id"]):
                link = HttpRequest.build_absolute_uri(
                    request, "/user/contacts/"
                )
                invite_decline(ui, link)
    status = 200
    return JsonResponse(response, status=status)


@ajax_required
@require_POST
def get_confirmkey_data(request):
    """
    Get data for an email confirmation key
    """
    response = {}
    key = request.POST["key"]
    confirmation = EmailConfirmationHMAC.from_key(key)
    if not confirmation:
        qs = EmailConfirmation.objects.all_valid()
        qs = qs.select_related("email_address__user")
        confirmation = qs.filter(key=key.lower()).first()
    if confirmation:
        status = 200
        response["username"] = confirmation.email_address.user.username
        response["email"] = confirmation.email_address.email
        if request.user:
            if request.user != confirmation.email_address.user:
                response["logout"] = True
                logout(request)
        # We check if the user has another verified email already. If yes,
        # we don't need to display the terms and test server warning again.
        if confirmation.email_address.user.emailaddress_set.filter(
            verified=True
        ).first():
            response["verified"] = True
        else:
            response["verified"] = False
    else:
        status = 404
    return JsonResponse(response, status=status)


class FidusSignupView(SignupView):
    def form_valid(self, form):
        if not settings.REGISTRATION_OPEN:
            return HttpResponseRedirect("/")
        ret = super().form_valid(form)
        if ret.status_code > 399:
            return ret
        if "invite_key" in self.request.POST:
            invites_connect(self.user, self.request.POST["invite_key"])
        return ret


signup = FidusSignupView.as_view()



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^save/$", views.save, name="bibliography_save"),
    re_path("^delete/$", views.delete, name="bibliography_delete"),
    re_path(
        "^save_category/$",
        views.save_category,
        name="bibliopgraphy_save_category",
    ),
    re_path(
        "^delete_category/$",
        views.delete_category,
        name="bibliography_delete_category",
    ),
    re_path("^biblist/$", views.biblist, name="bibliography_biblist"),
]



from django.conf import settings
from avatar.utils import get_username
from avatar.providers import PrimaryAvatarProvider


class Avatars:
    def __init__(self):
        self.AVATARS = {}

    def get_url(self, user):
        if user.id not in self.AVATARS:
            username = get_username(user)
            self.AVATARS[user.id] = PrimaryAvatarProvider.get_avatar_url(
                username, settings.AVATAR_DEFAULT_SIZE
            )
        return self.AVATARS[user.id]



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as DefaultUserAdmin
from django.utils.translation import gettext_lazy as _

from . import models


class UserAdmin(DefaultUserAdmin):
    fieldsets = DefaultUserAdmin.fieldsets + (
        (_("Connections"), {"fields": ("contacts",)}),
    )


admin.site.register(models.User, UserAdmin)


class UserInviteAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.UserInvite, UserInviteAdmin)



# Generated by Django 4.1.2 on 2023-01-29 22:16

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("website", "0008_alter_editor_user"),
    ]

    operations = [
        migrations.CreateModel(
            name="Design",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "style",
                    models.TextField(
                        default="\n:root {\n    --posts_per_page: 10; /* Number of posts per page on frontpage. Disable for all. */\n}\n",
                        help_text="The CSS style definiton.",
                    ),
                ),
                (
                    "site",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="sites.site",
                    ),
                ),
            ],
        ),
    ]



from django.conf import settings
from django.db import models
from django.contrib.sites.models import Site

from document.models import Document


STATUS_CHOICES = (
    ("unsubmitted", "Unsubmitted"),
    ("submitted", "Submitted"),
    ("published", "Published"),
    ("resubmitted", "Resubmitted"),
)


class Publication(models.Model):
    document = models.OneToOneField(
        Document,
        on_delete=models.deletion.CASCADE,
    )
    title = models.CharField(max_length=255, default="", blank=True)
    added = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    submitter = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.deletion.CASCADE,
    )
    status = models.CharField(
        choices=STATUS_CHOICES,
        max_length=11,
        default="unsubmitted",
    )
    messages = models.JSONField(default=list)
    authors = models.JSONField(default=list)
    keywords = models.JSONField(default=list)
    abstract = models.TextField(default="")

    html_src = models.TextField(
        default="",
    )  # The original HTML as exported from the frontend
    html_output = models.TextField(
        default="",
    )  # The HTML with asset locations replaced.

    def __str__(self):
        return f"{self.title} ({self.document_id}, {self.status})"


def publication_asset_location(instance, filename):
    # preserve the original filename
    instance.filename = filename
    return "/".join(["publication_assets", filename])


class PublicationAsset(models.Model):
    publication = models.ForeignKey(
        Publication,
        on_delete=models.deletion.CASCADE,
    )
    file = models.FileField(
        upload_to=publication_asset_location,
        help_text=(
            "A file references in the HTML of a Publication. The filepath "
            "will be replaced with the final url of the file in the style."
        ),
    )
    filepath = models.CharField(
        max_length=255,
        help_text="The original filepath.",
    )


default_style = """
:root {
    --posts_per_page: 10; /* Number of posts per page on frontpage. Disable for all. */
}
"""


class Design(models.Model):
    site = models.OneToOneField(Site, on_delete=models.deletion.CASCADE)
    style = models.TextField(
        help_text="The CSS style definiton.", default=default_style
    )

    def __str__(self):
        return f"{self.site.name}"


class Editor(models.Model):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.deletion.CASCADE,
        related_name="website_editor",
    )

    def __str__(self):
        return f"Editor {self.user.username}"



# Generated by Django 1.11.13 on 2018-08-14 17:32
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    replaces = [
        ("bibliography", "0001_initial"),
        ("bibliography", "0002_entry_bib_type"),
        ("bibliography", "0003_auto_20161113_1534"),
        ("bibliography", "0004_remove_entry_entry_type"),
        ("bibliography", "0005_auto_20161115_0310"),
        ("bibliography", "0006_auto_20161122_0304"),
        ("bibliography", "0007_auto_20161201_0209"),
        ("bibliography", "0008_auto_20161201_0545"),
        ("bibliography", "0009_auto_20161207_0130"),
        ("bibliography", "0010_auto_20161207_0133"),
        ("bibliography", "0011_auto_20170101_1647"),
    ]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Entry",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("entry_key", models.CharField(max_length=64)),
                ("entry_cat", models.CharField(default="", max_length=255)),
                ("last_modified", models.DateTimeField(auto_now=True)),
                ("fields", models.TextField(default="{}")),
                (
                    "entry_owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                ("bib_type", models.CharField(default="", max_length=30)),
            ],
        ),
        migrations.CreateModel(
            name="EntryCategory",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("category_title", models.CharField(max_length=100)),
                (
                    "category_owner",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Entry categories",
            },
        ),
        migrations.AlterUniqueTogether(
            name="entry",
            unique_together=set([]),
        ),
        migrations.AlterField(
            model_name="entry",
            name="entry_cat",
            field=models.TextField(default="[]"),
        ),
    ]



from django.contrib import admin
from . import models


class BookRepositoryAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.BookRepository, BookRepositoryAdmin)


class RepoInfoAdmin(admin.ModelAdmin):
    pass


admin.site.register(models.RepoInfo, RepoInfoAdmin)



from django.urls import re_path

from . import views

urlpatterns = [
    re_path("^get_book_repos/$", views.get_book_repos, name="get_book_repos"),
    re_path(
        "^update_book_repo/$", views.update_book_repo, name="update_book_repo"
    ),
]



# Generated by Django 3.2.13 on 2022-07-15 11:50

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0005_auto_20220715_1044"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="added",
            field=models.DateTimeField(
                auto_now_add=True,
                default=django.utils.timezone.now,
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="publication",
            name="updated",
            field=models.DateTimeField(auto_now=True),
        ),
    ]



# Generated by Django 4.1.2 on 2023-01-04 16:02

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("website", "0007_publication_abstract"),
    ]

    operations = [
        migrations.AlterField(
            model_name="editor",
            name="user",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="website_editor",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



from pathlib import Path
import shutil
from subprocess import call

from base.management import BaseCommand
from django.core.management import call_command
from django.conf import settings


BABEL_CONF = """
module.exports = {
    presets: [
        [
            '@babel/preset-env',
            {
                targets: {
                    node: 'current',
                },
            },
        ],
    ],
}
"""


class Command(BaseCommand):
    help = "Run jest unit tests."

    def handle(self, *ars, **options):
        call_command("transpile")
        p = Path(settings.PROJECT_PATH) / ".transpile"
        shutil.os.chdir(p)
        conf_file = p / "babel.config.js"

        if not conf_file.exists():
            self.stdout.write(f'Creating "babel.config.js" at {p}.')
            conf_file.write_text(BABEL_CONF)

        command_array = [
            p / "node_modules" / ".bin" / "jest",
            "--no-cache",
            "--passWithNoTests",
        ]
        return_value = call(command_array)
        if return_value > 0:
            exit(return_value)



# Generated by Django 3.1.4 on 2021-05-25 20:22

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("document", "0013_auto_20210406_0640"),
        ("user", "0006_userinvite"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accessright",
            name="holder_type",
            field=models.ForeignKey(
                limit_choices_to=models.Q(
                    models.Q(("app_label", "user"), ("model", "user")),
                    models.Q(("app_label", "user"), ("model", "userinvite")),
                    _connector="OR",
                ),
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
        ),
    ]



from django.urls import re_path

from . import views

urlpatterns = [
    re_path(
        "^documentlist/$", views.get_documentlist, name="get_documentlist"
    ),
    re_path(
        "^documentlist/extra/$",
        views.get_documentlist_extra,
        name="get_documentlist_extra",
    ),
    re_path("^delete/$", views.delete, name="delete"),
    re_path("^move/$", views.move, name="move"),
    re_path("^create_doc/$", views.create_doc, name="create_doc"),
    re_path("^import/create/$", views.import_create, name="import_create"),
    re_path("^import/image/$", views.import_image, name="import_image"),
    re_path("^import/$", views.import_doc, name="import_doc"),
    re_path("^upload/$", views.upload_revision, name="upload_revision"),
    re_path(
        "^get_revision/(?P<revision_id>[0-9]+)/$",
        views.get_revision,
        name="get_revision",
    ),
    re_path(
        "^delete_revision/$", views.delete_revision, name="delete_revision"
    ),
    re_path(
        "^get_access_rights/$",
        views.get_access_rights,
        name="get_access_rights",
    ),
    re_path(
        "^save_access_rights/$",
        views.save_access_rights,
        name="save_access_rights",
    ),
    re_path("^comment_notify/$", views.comment_notify, name="comment_notify"),
    re_path(
        "^admin/get_template/$",
        views.get_template_admin,
        name="get_template_admin",
    ),
    re_path(
        "^get_template_for_doc/$",
        views.get_template_for_doc,
        name="get_template_for_doc",
    ),
    re_path(
        "^admin/create_template/$",
        views.create_template_admin,
        name="create_template_admin",
    ),
    re_path(
        "^admin/get_template/(?P<type>base|extras)/$",
        views.get_template_admin,
        name="get_template_admin",
    ),
    re_path(
        "^admin/get_all_old/$", views.get_all_old_docs, name="get_all_old_docs"
    ),
    re_path("^admin/save_doc/$", views.save_doc, name="save_doc"),
    re_path(
        "^admin/add_images_to_doc/$",
        views.add_images_to_doc,
        name="add_images_to_doc",
    ),
    re_path(
        "^admin/get_all_revision_ids/$",
        views.get_all_revision_ids,
        name="get_all_revision_ids",
    ),
    re_path(
        "^admin/get_all_template_ids/$",
        views.get_all_template_ids,
        name="get_all_template_ids",
    ),
    re_path(
        "^admin/save_template/$", views.save_template, name="save_template"
    ),
    re_path(
        "^admin/get_user_biblist/$",
        views.get_user_biblist,
        name="get_user_biblist",
    ),
    re_path(
        "^admin/update_revision/$",
        views.update_revision,
        name="update_revision",
    ),
]



from builtins import object
from django.forms import ModelForm
from django.contrib.auth import get_user_model


class UserForm(ModelForm):
    class Meta(object):
        model = get_user_model()
        fields = ("username", "first_name", "last_name")



from django.db import models
from django.conf import settings as django_settings
from book.models import Book

REPO_TYPES = (
    ("github", "GitHub"),
    ("gitlab", "GitLab"),
)


class BookRepository(models.Model):
    book = models.ForeignKey(Book, on_delete=models.deletion.CASCADE)
    repo_id = models.IntegerField()
    repo_name = models.CharField(
        max_length=256,
    )
    repo_type = models.CharField(
        max_length=6,
        choices=REPO_TYPES,
    )
    export_epub = models.BooleanField()
    export_unpacked_epub = models.BooleanField()
    export_html = models.BooleanField()
    export_unified_html = models.BooleanField()
    export_latex = models.BooleanField()

    class Meta(object):
        verbose_name_plural = "Book repositories"


class RepoInfo(models.Model):
    user = models.OneToOneField(
        django_settings.AUTH_USER_MODEL,
        on_delete=models.deletion.CASCADE,
    )
    content = models.JSONField(default=list, null=True, blank=True)

    def __str__(self):
        return self.user.readable_name



# Generated by Django 3.2.4 on 2021-11-26 07:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("book", "0012_remove_user_accessrights"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="RepoInfo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "content",
                    models.JSONField(blank=True, default=list, null=True),
                ),
                (
                    "user",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="BookRepository",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("repo_id", models.IntegerField()),
                ("repo_name", models.CharField(max_length=256)),
                (
                    "repo_type",
                    models.CharField(
                        choices=[("github", "GitHub"), ("gitlab", "GitLab")],
                        max_length=6,
                    ),
                ),
                ("export_epub", models.BooleanField()),
                ("export_unpacked_epub", models.BooleanField()),
                ("export_html", models.BooleanField()),
                ("export_unified_html", models.BooleanField()),
                ("export_latex", models.BooleanField()),
                (
                    "book",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="book.book",
                    ),
                ),
            ],
            options={
                "verbose_name_plural": "Book repositories",
            },
        ),
    ]



# Generated by Django 3.2.13 on 2022-07-07 17:55

from django.db import migrations, models
import django.db.models.deletion
import website.models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0002_auto_20220703_1201"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="html_output",
            field=models.TextField(default=""),
        ),
        migrations.AddField(
            model_name="publication",
            name="html_src",
            field=models.TextField(default=""),
        ),
        migrations.CreateModel(
            name="PublicationAsset",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "file",
                    models.FileField(
                        help_text="A file references in the HTML of a Publication. The filepath will be replaced with the final url of the file in the style.",
                        upload_to=website.models.publication_asset_location,
                    ),
                ),
                (
                    "filepath",
                    models.CharField(
                        help_text="The original filepath.",
                        max_length=255,
                    ),
                ),
                (
                    "publication",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="website.publication",
                    ),
                ),
            ],
        ),
    ]



import re
import json

from tornado.httpclient import AsyncHTTPClient, HTTPRequest, HTTPError
from allauth.socialaccount.models import SocialToken

ALLOWED_PATHS = [
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/contents/"),
    re.compile(r"^user/repos$"),
    re.compile(r"^user/repos/reload$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/blobs/([\w\d]+)$"),
    re.compile(
        r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/refs/heads/([\w\d]+)$"
    ),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/blobs$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/commits$"),
    re.compile(r"^repos/([\w\.\-@_]+)/([\w\.\-@_]+)/git/trees$"),
]

ALLOWED_METHODS = ["GET", "POST", "PATCH"]


def get_headers(token):
    return {
        "Authorization": f"token {token}",
        "User-Agent": "Fidus Writer",
        "Accept": "application/vnd.github.v3+json",
    }


async def prepare(proxy_connector, path_parts, user):
    path = "/".join(path_parts)
    method = proxy_connector.request.method
    if not any(regex.match(path) for regex in ALLOWED_PATHS):
        proxy_connector.set_status(401)
        return
    if method not in ALLOWED_METHODS:
        proxy_connector.set_status(405)
        return
    social_token = SocialToken.objects.filter(
        account__user=user, account__provider="github"
    ).first()
    if not social_token:
        proxy_connector.set_status(401)
        return
    headers = get_headers(social_token.token)
    query = proxy_connector.request.query
    url = f"https://api.github.com/{path}"
    if query:
        url += "?" + query
    if method == "GET":
        body = None
    else:
        body = proxy_connector.request.body
    request = HTTPRequest(url, method, headers, body=body, request_timeout=120)
    http = AsyncHTTPClient()
    try:
        response = await http.fetch(request)
    except HTTPError as e:
        if e.response.code == 404:
            # We remove the 404 response so it will not show up as an
            # error in the browser
            proxy_connector.write("[]")
        else:
            proxy_connector.set_status(e.response.code)
            proxy_connector.write(e.response.body)
    except Exception as e:
        proxy_connector.set_status(500)
        proxy_connector.write("Error: %s" % e)
    else:
        proxy_connector.set_status(response.code)
        proxy_connector.write(response.body)


def githubrepo2repodata(github_repo):
    return {
        "type": "github",
        "name": github_repo["full_name"],
        "id": github_repo["id"],
        "branch": github_repo["default_branch"],
    }


async def get_repos(proxy_connector, github_token):
    headers = get_headers(github_token)
    repos = []
    page = 1
    last_page = False
    while not last_page:
        url = f"https://api.github.com/user/repos?page={page}&per_page=100"
        request = HTTPRequest(url, "GET", headers, request_timeout=120)
        http = AsyncHTTPClient()
        try:
            response = await http.fetch(request)
        except HTTPError as e:
            if e.response.code == 404:
                # We remove the 404 response so it will not show up as an
                # error in the browser
                return []
            else:
                proxy_connector.set_status(e.response.code)
                proxy_connector.write(e.response.body)
            return []
        except Exception as e:
            proxy_connector.set_status(500)
            proxy_connector.write("Error: %s" % e)
            return []
        else:
            content = json.loads(response.body)
            repos += map(githubrepo2repodata, content)
            if len(content) == 100:
                page += 1
            else:
                last_page = True
    return repos



# Generated by Django 3.2.13 on 2022-07-11 12:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("website", "0003_auto_20220707_1955"),
    ]

    operations = [
        migrations.AddField(
            model_name="publication",
            name="title",
            field=models.CharField(blank=True, default="", max_length=255),
        ),
    ]



import json
from django.db import migrations, models, transaction


def text_to_json(apps, schema_editor):
    Document = apps.get_model("document", "Document")
    documents = Document.objects.all().iterator()
    with transaction.atomic():
        for document in documents:
            document.content = json.loads(document.contents)
            document.diffs = json.loads(document.last_diffs)
            document.comments = json.loads(document.comments_text)
            document.bibliography = json.loads(document.bibliography_text)
            for field in document._meta.local_fields:
                if field.name == "updated":
                    field.auto_now = False
            document.save()
    DocumentTemplate = apps.get_model("document", "DocumentTemplate")
    templates = DocumentTemplate.objects.all()
    with transaction.atomic():
        for template in templates:
            template.content = json.loads(template.definition)
            template.save()


def json_to_text(apps, schema_editor):
    Document = apps.get_model("document", "Document")
    documents = Document.objects.all().iterator()
    with transaction.atomic():
        for document in documents:
            document.contents = json.dumps(document.content)
            document.last_diffs = json.dumps(document.diffs)
            document.comments_text = json.dumps(document.comments)
            document.bibliography_text = json.dumps(document.bibliography)
            for field in document._meta.local_fields:
                if field.name == "updated":
                    field.auto_now = False
            document.save()
    DocumentTemplate = apps.get_model("document", "DocumentTemplate")
    templates = DocumentTemplate.objects.all()
    with transaction.atomic():
        for template in templates:
            template.definition = json.dumps(template.content)
            template.save()


class Migration(migrations.Migration):
    atomic = False  # to prevent migrations running in single transaction
    dependencies = [
        ("document", "0003_fidus_3_3"),
    ]

    operations = [
        migrations.RenameField(
            model_name="document",
            old_name="comments",
            new_name="comments_text",
        ),
        migrations.AddField(
            model_name="document",
            name="comments",
            field=models.JSONField(default=dict),
        ),
        migrations.RenameField(
            model_name="document",
            old_name="bibliography",
            new_name="bibliography_text",
        ),
        migrations.AddField(
            model_name="document",
            name="bibliography",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="document",
            name="content",
            field=models.JSONField(default=dict),
        ),
        migrations.AddField(
            model_name="document",
            name="diffs",
            field=models.JSONField(default=list),
        ),
        migrations.AddField(
            model_name="documenttemplate",
            name="content",
            field=models.JSONField(default=dict),
        ),
        migrations.RunPython(text_to_json, json_to_text),
        migrations.RemoveField(
            model_name="document",
            name="comments_text",
        ),
        migrations.RemoveField(
            model_name="document",
            name="bibliography_text",
        ),
        migrations.RemoveField(
            model_name="document",
            name="contents",
        ),
        migrations.RemoveField(
            model_name="document",
            name="last_diffs",
        ),
        migrations.RemoveField(
            model_name="documenttemplate",
            name="definition",
        ),
    ]



import time
import zipfile

from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.shortcuts import get_current_site
from django.core.files.base import ContentFile
from django.http import HttpRequest
from django.http import HttpResponse
from django.http import JsonResponse
from django.views.decorators.http import require_POST
from document.models import AccessRight
from document.models import Document
from django.core.paginator import Paginator

from . import emails
from . import models


@login_required
@require_POST
def get_doc_info(request):
    response = {}
    document_id = int(request.POST.get("doc_id"))
    document = Document.objects.filter(id=document_id).first()
    if not document:
        return HttpResponse("Not found", status=404)
    if (
        document.owner != request.user
        and not AccessRight.objects.filter(
            document=document,
            user=request.user,
        ).first()
    ):
        # Access forbidden
        return HttpResponse("Missing access rights", status=403)
    response["submission"] = {}
    publication = models.Publication.objects.filter(
        document_id=document_id,
    ).first()
    if publication:
        response["submission"]["status"] = publication.status
        response["submission"]["messages"] = publication.messages
    else:
        response["submission"]["status"] = "unsubmitted"
        response["submission"]["messages"] = []
    if request.user.has_perm(
        "website.add_publication",
    ) or models.Editor.objects.filter(
        user_id=request.user.id,
    ):
        user_role = "editor"
    else:
        user_role = "author"
    response["submission"]["user_role"] = user_role
    status = 200
    return JsonResponse(response, status=status)


@login_required
@require_POST
def submit_doc(request):
    response = {}
    document_id = int(request.POST.get("doc_id"))
    status = 200
    document = Document.objects.filter(id=document_id).first()
    if not document:
        return HttpResponse("Not found", status=404)
    if (
        document.owner != request.user
        and not AccessRight.objects.filter(
            document=document,
            user=request.user,
        ).first()
    ):
        # Access forbidden
        return HttpResponse("Missing document access rights", status=403)
    publication, created = models.Publication.objects.get_or_create(
        document_id=document_id,
        defaults={"submitter": request.user, "status": "submitted"},
    )
    if (
        publication.status == "published"
        and not created
        and request.user.has_perm("website.change_publication")
    ):
        # The user has permission to publish the document immediately.
        publication.title = request.POST.get("title")
        publication.abstract = request.POST.get("abstract")
        publication.authors = request.POST.getlist("authors[]")
        publication.keywords = request.POST.getlist("keywords[]")
        # Delete all existing assets
        models.PublicationAsset.objects.filter(
            publication=publication,
        ).delete()
        html_zip = zipfile.ZipFile(request.FILES.get("html.zip"))
        body_html = html_zip.open("document.html").read().decode("utf-8")
        publication.html_src = body_html
        publication.status = "published"
        message = {
            "type": "publish",
            "message": request.POST.get("message"),
            "user": request.user.readable_name,
            "time": time.time(),
        }
        publication.messages.append(message)
        response["message"] = message
        publication.save()
        response["status"] = publication.status
        return JsonResponse(response, status=status)
    message = {
        "type": "submit",
        "message": request.POST.get("message"),
        "user": request.user.readable_name,
        "time": time.time(),
    }
    publication.messages.append(message)
    response["message"] = message
    if not created:
        if publication.status in ["published", "resubmitted"]:
            publication.status = "resubmitted"
        else:
            publication.status = "submitted"
    publication.save()
    link = HttpRequest.build_absolute_uri(request, document.get_absolute_url())
    user_ct = ContentType.objects.get(app_label="user", model="user")
    for editor in models.Editor.objects.select_related("user").all():
        if editor.user == document.owner or request.user == editor.user:
            continue
        access_right, created = AccessRight.objects.get_or_create(
            document_id=document_id,
            holder_id=editor.user.id,
            holder_type=user_ct,
            defaults={
                "rights": "write",
            },
        )
        if not created and access_right.rights != "write":
            access_right.rights = "write"
            access_right.save()
        if editor.user != request.user:
            emails.send_submit_notification(
                document.title,
                link,
                request.POST.get("message"),
                editor.user.readable_name,
                editor.user.email,
            )

    response["status"] = publication.status
    return JsonResponse(response, status=status)


@login_required
@require_POST
def reject_doc(request):
    response = {}
    if not (
        request.user.has_perm("website.add_publication")
        or request.user.has_perm("website.change_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        # Access forbidden
        return HttpResponse("Missing access rights", status=403)
    document_id = int(request.POST.get("doc_id"))
    document = Document.objects.filter(id=document_id).first()
    if not document:
        return HttpResponse("Not found", status=404)
    if (
        document.owner != request.user
        and not AccessRight.objects.filter(
            document=document,
            user=request.user,
        ).first()
    ):
        # Access forbidden
        return HttpResponse("Missing document access rights", status=403)
    status = 200
    if (
        request.user.has_perm("website.add_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        publication, created = models.Publication.objects.get_or_create(
            document_id=document_id,
            defaults={"submitter": request.user, "status": "rejected"},
        )
    else:
        publication = models.Publication.objects.filter(
            document_id=document_id,
        ).first()
        if not publication:
            # Access forbidden
            return HttpResponse("Missing document access rights", status=403)
        created = False
    if not created:
        publication.status = "rejected"
    message = {
        "type": "reject",
        "message": request.POST.get("message"),
        "user": request.user.readable_name,
        "time": time.time(),
    }
    publication.messages.append(message)
    publication.save()
    response["message"] = message
    response["status"] = publication.status
    if document.owner != request.user:
        emails.send_reject_notification(
            document.title,
            HttpRequest.build_absolute_uri(
                request, document.get_absolute_url()
            ),
            request.POST.get("message"),
            document.owner.readable_name,
            document.owner.email,
        )
    return JsonResponse(response, status=status)


@login_required
@require_POST
def review_doc(request):
    response = {}
    if not (
        request.user.has_perm("website.add_publication")
        or request.user.has_perm("website.change_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        # Access forbidden
        return HttpResponse("Missing access rights", status=403)
    document_id = int(request.POST.get("doc_id"))
    document = Document.objects.filter(id=document_id).first()
    if not document:
        return HttpResponse("Not found", status=404)
    if (
        document.owner != request.user
        and not AccessRight.objects.filter(
            document=document,
            user=request.user,
        ).first()
    ):
        # Access forbidden
        return HttpResponse("Missing access rights", status=403)
    status = 200
    if (
        request.user.has_perm("website.add_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        publication, _created = models.Publication.objects.get_or_create(
            document_id=document_id,
            defaults={"submitter": request.user, "status": "unsubmitted"},
        )
    else:
        publication = models.Publication.objects.filter(
            document_id=document_id,
        ).first()
        if not publication:
            # Access forbidden
            return HttpResponse("Missing access rights", status=403)
    message = {
        "type": "review",
        "message": request.POST.get("message"),
        "user": request.user.readable_name,
        "time": time.time(),
    }
    publication.messages.append(message)
    publication.save()
    response["message"] = message
    if document.owner != request.user:
        emails.send_review_notification(
            document.title,
            HttpRequest.build_absolute_uri(
                request, document.get_absolute_url()
            ),
            request.POST.get("message"),
            document.owner.readable_name,
            document.owner.email,
        )
    return JsonResponse(response, status=status)


@login_required
@require_POST
def publish_doc(request):
    response = {}
    if not (
        request.user.has_perm("website.add_publication")
        or request.user.has_perm("website.change_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        # Access forbidden
        return HttpResponse("Missing access rights", status=403)
    document_id = int(request.POST.get("doc_id"))
    document = Document.objects.filter(id=document_id).first()
    if not document:
        return HttpResponse("Not found", status=404)
    if (
        document.owner != request.user
        and not AccessRight.objects.filter(
            document=document,
            user=request.user,
            rights="write",
        ).first()
    ):
        # Access forbidden
        return HttpResponse("Missing document access rights", status=403)
    if (
        request.user.has_perm("website.add_publication")
        or models.Editor.objects.filter(user=request.user).first()
    ):
        publication, created = models.Publication.objects.get_or_create(
            document_id=document_id,
            defaults={"submitter_id": request.user.id},
        )
    else:
        publication = models.Publication.objects.filter(
            document_id=document_id,
        )
        if not publication:
            # Access forbidden
            return HttpResponse("Missing access rights", status=403)
    publication.title = request.POST.get("title")
    publication.abstract = request.POST.get("abstract")
    publication.authors = request.POST.getlist("authors[]")
    publication.keywords = request.POST.getlist("keywords[]")
    # Delete all existing assets
    models.PublicationAsset.objects.filter(publication=publication).delete()
    html_zip = zipfile.ZipFile(request.FILES.get("html.zip"))
    body_html = html_zip.open("document.html").read().decode("utf-8")
    publication.html_src = body_html
    publication.status = "published"
    message = {
        "type": "publish",
        "message": request.POST.get("message"),
        "user": request.user.readable_name,
        "time": time.time(),
    }
    publication.messages.append(message)
    response["message"] = message
    publication.save()

    # Iterate over document files
    for filepath in html_zip.namelist():
        if (
            filepath.endswith("/")
            or filepath == "document.html"
            or filepath not in body_html
        ):
            continue
        file = ContentFile(
            html_zip.open(filepath).read(),
            name=filepath.split("/")[-1],
        )
        asset = models.PublicationAsset.objects.create(
            publication=publication,
            file=file,
            filepath=filepath,
        )
        body_html = body_html.replace(filepath, asset.file.url)

    # Save html with adjusted links to media files with publication.
    publication.html_output = body_html
    publication.save()

    response["status"] = publication.status
    status = 200
    if document.owner != request.user:
        emails.send_publish_notification(
            document.title,
            HttpRequest.build_absolute_uri(
                request, document.get_absolute_url()
            ),
            request.POST.get("message"),
            document.owner.readable_name,
            document.owner.email,
        )
    return JsonResponse(response, status=status)


def list_publications(request, per_page=False, page_number=1):
    publications = models.Publication.objects.filter(
        status="published",
    ).order_by(
        "-added",
    )
    response = {}
    if per_page:
        paginator = Paginator(publications, per_page)
        page = paginator.get_page(page_number)
        publications = page.object_list
        response["num_pages"] = paginator.num_pages

    response["publications"] = [
        {
            "title": pub.title,
            "abstract": pub.abstract,
            "keywords": pub.keywords,
            "authors": pub.authors,
            "id": pub.id,
            "doc_id": pub.document_id,
            "added": pub.added,
            "updated": pub.updated,
        }
        for pub in publications
    ]
    response["site_name"] = get_current_site(request).name
    return JsonResponse(response, status=200)


def get_publication(request, id):
    publication = models.Publication.objects.filter(id=id).first()
    response = {}
    if publication:
        response["title"] = publication.title
        response["content"] = publication.html_output

        document = publication.document
        if not request.user.is_anonymous and (
            document.owner == request.user
            or AccessRight.objects.filter(
                document=document,
                user=request.user,
            ).first()
        ):
            # Has access right
            response["can_edit"] = True
        else:
            response["can_edit"] = False
        response["doc_id"] = publication.document_id

    return JsonResponse(response, status=200)


def get_style(request):
    response = {}
    design = models.Design.objects.filter(
        site__id=get_current_site(request).id
    ).first()
    if design:
        response["style"] = design.style
    return JsonResponse(response, status=200)
